unit NetDevices;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Grids, StdCtrls, RinNMUDP, MSXML2_TLB, ComObj, ActiveX, Menus, ShellApi,
  Winsock2;

const
  IpBCastSeparator = ' / ';

type

  TRinstrumDevice = class(TOBject)
  protected
    RinDeviceIPAddress : String;
    RinDeviceClass : String;
    RinDeviceName : String;
    RinDeviceSerialNumber : String;
    RinDeviceVersion : String;
    RinDeviceMacAddress : String;
    RinDeviceCustomId : String;
    RinDeviceTcpPort : String;
    RinDeviceHostname : String;
  public
    constructor Create;

    procedure AddToStringGrid(sg : TStringGrid);
    procedure ExtractFromXML(xml : String; IpAddress : String);
  end;

  TfrmNetworkDevices = class(TForm)
    GrpBoxDevices: TGroupBox;
    StringGridNetDevices: TStringGrid;
    BtnFind: TButton;
    BtnClose: TButton;
    LabelIPAddress: TLabel;
    PopupMenuDeviceClick: TPopupMenu;
    PMConnect: TMenuItem;
    BtnConnect: TButton;
    BtnWeb: TButton;
    CBInterfaces: TComboBox;
    procedure FormCreate(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure BtnCloseClick(Sender: TObject);
    procedure BtnFindClick(Sender: TObject);
    procedure NMUDPSockInvalidHost(var handled: Boolean);
    procedure NMUDPSockDataReceived(Sender: TComponent;
      NumberBytes: Integer; FromIP: String; Port: Integer);
    procedure StringGridNetDevicesSelectCell(Sender: TObject; ACol,
      ARow: Integer; var CanSelect: Boolean);
    procedure BtnWebClick(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
  private
    { Private declarations }
    NMUDPSock : TRinNMUDP;
    FCanConnect : Boolean;
    procedure SetCanConnect(value : Boolean);
    function GetConnectAddress : String;
    function GetConnectTcpPort : String;
    function GetLookupAddress : String;
    procedure SetLookupAddress(value : String);
    procedure ResizeColumns;
    procedure ResizeComponents;
    procedure PopulateInterfacesList;
    function GetInterfaceForBroadcast(BCastIp : String) : String;
    procedure InitiateDeviceFind(const ipaddress : String; const localbindip : String);
  public
    { Public declarations }
    property CanConnect : Boolean read FCanConnect write SetCanConnect;
    property LookUpAddress : String read GetLookupAddress write SetLookupAddress;
    property ConnectAddress : String read GetConnectAddress;
    property ConnectTcpPort : String read GetConnectTcpPort;
  end;

var
  frmNetworkDevices: TfrmNetworkDevices;

implementation

{$R *.DFM}


constructor TRinstrumDevice.Create;
begin
  inherited Create;

  Self.RinDeviceIPAddress := '';
  Self.RinDeviceClass := '';
  Self.RinDeviceName := '';
  Self.RinDeviceSerialNumber := '';
  Self.RinDeviceVersion := '';
  Self.RinDeviceMacAddress := '';
  Self.RinDeviceCustomId := '';
  Self.RinDeviceTcpPort := '';
  Self.RinDeviceHostname := '';
end;


procedure TRinstrumDevice.AddToStringGrid(sg : TStringGrid);
var rc : Integer;
begin
  sg.RowCount := sg.RowCount + 1;
  rc := sg.RowCount - 1;
  sg.Cells[0, rc] := Self.RinDeviceIPAddress;
  sg.Cells[1, rc] := Self.RinDeviceClass;
  sg.Cells[2, rc] := Self.RinDeviceName;
  sg.Cells[3, rc] := Self.RinDeviceSerialNumber;
  sg.Cells[4, rc] := Self.RinDeviceVersion;
  sg.Cells[5, rc] := Self.RinDeviceCustomId;
  sg.Cells[6, rc] := Self.RinDeviceMacAddress;
  sg.Cells[7, rc] := Self.RinDeviceTcpPort;
  sg.Cells[8, rc] := Self.RinDeviceHostname;
end;


procedure TRinstrumDevice.ExtractFromXML(xml : String; IpAddress : String);
var
  i : Integer;
  rinNodes : IXMLDOMNodeList;
//  subNodes : IXMLDOMNodeList;
//  node : IXMLDOMElement;
  FDocument : IXMLDOMDocument;
begin

  OleCheck(CoCreateInstance(Class_DOMDocument, nil, CLSCTX_ALL,IXMLDOMDocument, FDocument));
  if FDocument = nil then
  begin
    Raise Exception.Create('Could not create XML doc');
  end;
  FDocument.loadXML(xml);

  rinNodes := FDocument.selectNodes('//rinstrum//device//*');
  if (rinNodes <> nil) then
  begin
    for i := 0 to rinNodes.length - 1 do
    begin
      if ('class' = rinNodes.item[i].nodeName) then
        Self.RinDeviceClass := rinNodes.item[i].text
      else if ('name' = rinNodes.item[i].nodeName) then
        Self.RinDeviceName := rinNodes.item[i].text
      else if ('serialnumber' = rinNodes.item[i].nodeName) then
        Self.RinDeviceSerialNumber := rinNodes.item[i].text
      else if ('version' = rinNodes.item[i].nodeName) then
        Self.RinDeviceVersion := rinNodes.item[i].text
      else if ('customid' = rinNodes.item[i].nodeName) then
        Self.RinDeviceCustomId := rinNodes.item[i].text
      else if ('macaddress' = rinNodes.item[i].nodeName) then
        Self.RinDeviceMacAddress := rinNodes.item[i].text
      else if ('tcpport' = rinNodes.item[i].nodeName) then
        Self.RinDeviceTcpPort := rinNodes.item[i].text
      else if ('hostname' = rinNodes.item[i].nodeName) then
        Self.RinDeviceHostname := rinNodes.item[i].text
    end;
  end;

  Self.RinDeviceIPAddress := IpAddress;

//  FDocument._Release;
//  FDocument := nil;
end;


procedure TfrmNetworkDevices.FormCreate(Sender: TObject);
begin
  Randomize();
  NMUDPSock := TRinNMUDP.Create(nil);
  NMUDPSock.OnInvalidHost := NMUDPSockInvalidHost;
  NMUDPSock.OnDataReceived := NMUDPSockDataReceived;
  NMUDPSock.LocalHost := '0.0.0.0';
  NMUDPSock.LocalPort := Random(65535 - 1024) + 1024;
  Self.LookupAddress := '192.168.0.255';

  SetCanConnect(True);
  BtnConnect.Enabled := False;
  BtnWeb.Enabled := False;

  Self.StringGridNetDevices.ColCount := 9;
  Self.StringGridNetDevices.Cells[0, 0] := 'IP Address';
  Self.StringGridNetDevices.Cells[1, 0] := 'Device Class';
  Self.StringGridNetDevices.Cells[2, 0] := 'Device';
  Self.StringGridNetDevices.Cells[3, 0] := 'Serial Number';
  Self.StringGridNetDevices.Cells[4, 0] := 'Software Version';
  Self.StringGridNetDevices.Cells[5, 0] := 'Custom Identifier';
  Self.StringGridNetDevices.Cells[6, 0] := 'MAC Address';
  Self.StringGridNetDevices.Cells[7, 0] := 'TCP Port';
  Self.StringGridNetDevices.Cells[8, 0] := 'Hostname';
  Self.StringGridNetDevices.RowCount := 1;

  ResizeColumns();
  PopulateInterfacesList();
end;


procedure TfrmNetworkDevices.PopulateInterfacesList;
var sd : TSocket;
    InterfaceList : Array[0..20] of INTERFACE_INFO;
    nBytesReturned : Longint;
    nNumInterfaces : Longint;
    i : Longint;
    str : String;
    pAddress : PSockAddrIn;
    pNetmask : PSockAddrIn;
    nFlags : Longword;
    BCast : Longword;
    BCastStr: String;
begin
  sd := WSASocket(AF_INET, SOCK_DGRAM, 0, nil, 0, 0);
  if (Integer(sd) = SOCKET_ERROR) then
  begin
    str := Format('Failed to get a socket from WSASocket: Error %d.', [WSAGetLastError()]);
    MessageDlg(str, mtError, [mbOk], 0);
    Exit;
  end;

  if (WSAIoctl(sd, SIO_GET_INTERFACE_LIST, nil, 0, @InterfaceList,
               SizeOf(InterfaceList), @nBytesReturned, nil, nil) = SOCKET_ERROR) then
  begin
    str := Format('Failed calling WSAIoctl: Error %d.', [WSAGetLastError()]);
    MessageDlg(str, mtError, [mbOk], 0);
    Exit;
  end;

  nNumInterfaces := nBytesReturned div SizeOf(INTERFACE_INFO);
  //str := Format('There are %d interfaces.', [nNumInterfaces]);
  //MessageDlg(str, mtInformation, [mbOk], 0);

  CBInterfaces.Items.Clear();
  for i := 0 to nNumInterfaces - 1 do
  begin
    nFlags := InterfaceList[i].iiFlags;
    if (((nFlags and IFF_UP) <> 0) and
        ((nFlags and IFF_LOOPBACK) = 0)) then
    begin
      // Only list interfaces that are up and not loopback
      pAddress := PSockAddrIn(@(InterfaceList[i].iiAddress));
      pNetmask := PSockAddrIn(@(InterfaceList[i].iiNetmask));

      BCast := Longword(pAddress^.sin_addr) and Longword(pNetmask^.sin_addr);
      BCast := BCast or not Longword(pNetmask^.sin_addr);

      BCastStr := inet_ntoa(TInAddr(BCast));
      str := inet_ntoa(pAddress^.sin_addr) + IpBCastSeparator + BCastStr;
      CBInterfaces.Items.Add(str);
    end;

    {
    str := '';
    pAddress := PSockAddrIn(@(InterfaceList[i].iiAddress));
    str := str + Format('Address: %s, ', [inet_ntoa(pAddress^.sin_addr)]);

    pAddress := PSockAddrIn(@(InterfaceList[i].iiBroadcastAddress));
    str := str + Format('Broadcast: %s, ', [inet_ntoa(pAddress^.sin_addr)]);

    pAddress := PSockAddrIn(@(InterfaceList[i].iiNetmask));
    str := str + Format('Netmask: %s, ', [inet_ntoa(pAddress^.sin_addr)]);

    str := str + ' Iface is ';
    nFlags := InterfaceList[i].iiFlags;
    if ((nFlags and IFF_UP) <> 0) then
      str := str + 'up'
    else
      str := str + 'down';
    if ((nFlags and IFF_POINTTOPOINT) <> 0) then
      str := str + ', is point-to-point';
    if ((nFlags and IFF_LOOPBACK) <> 0) then
      str := str + ', is a loopback iface';
    str := str + ', and can do: ';
    if ((nFlags and IFF_BROADCAST) <> 0) then
      str := str + 'bcast ';
    if ((nFlags and IFF_MULTICAST) <> 0) then
      str := str + 'multicast ';

    MessageDlg(str, mtInformation, [mbOk], 0);
    }
  end;
end;


// Find an interface IP address
function TfrmNetworkDevices.GetInterfaceForBroadcast(BCastIp : String) : String;
var sd : TSocket;
    InterfaceList : Array[0..20] of INTERFACE_INFO;
    nBytesReturned : Longint;
    nNumInterfaces : Longint;
    i : Longint;
    str : String;
    pAddress : PSockAddrIn;
    pNetmask : PSockAddrIn;
    nFlags : Longword;
    BCast : Longword;
    Address : Longword;
    Netmask : Longword;
    AddressStr : String;
begin
  BCast := inet_addr(PChar(BCastIp));

  sd := WSASocket(AF_INET, SOCK_DGRAM, 0, nil, 0, 0);
  if (Integer(sd) = SOCKET_ERROR) then
  begin
    str := Format('Failed to get a socket from WSASocket: Error %d.', [WSAGetLastError()]);
    MessageDlg(str, mtError, [mbOk], 0);
    Exit;
  end;

  if (WSAIoctl(sd, SIO_GET_INTERFACE_LIST, nil, 0, @InterfaceList,
               SizeOf(InterfaceList), @nBytesReturned, nil, nil) = SOCKET_ERROR) then
  begin
    str := Format('Failed calling WSAIoctl: Error %d.', [WSAGetLastError()]);
    MessageDlg(str, mtError, [mbOk], 0);
    Exit;
  end;

  nNumInterfaces := nBytesReturned div SizeOf(INTERFACE_INFO);

  AddressStr := '0.0.0.0';
  for i := 0 to nNumInterfaces - 1 do
  begin
    nFlags := InterfaceList[i].iiFlags;
    if ((nFlags and IFF_UP) <> 0) then
    begin
      // Only list interfaces that are up and not loopback
      pAddress := PSockAddrIn(@(InterfaceList[i].iiAddress));
      pNetmask := PSockAddrIn(@(InterfaceList[i].iiNetmask));
      Address := Longword(pAddress^.sin_addr);
      Netmask := Longword(pNetmask^.sin_addr);

      if ((Address and Netmask) = (BCast and Netmask)) then
      begin
        AddressStr := inet_ntoa(pAddress^.sin_addr);
        break;
      end;
    end;
  end;
  Result := AddressStr;
end;


procedure TfrmNetworkDevices.SetCanConnect(value : Boolean);
begin
  if (not value) then
  begin
    BtnConnect.Enabled := False;
    BtnWeb.Enabled := False;
  end;
  FCanConnect := value;
end;


function TfrmNetworkDevices.GetConnectAddress : String;
begin
  if (BtnConnect.Enabled) then
  begin
    Result := Self.StringGridNetDevices.Cells[0, Self.StringGridNetDevices.Row];
  end
  else
  begin
    Result := '';
  end;
end;


function TfrmNetworkDevices.GetConnectTcpPort : String;
begin
  if (BtnConnect.Enabled) then
  begin
    Result := Self.StringGridNetDevices.Cells[7, Self.StringGridNetDevices.Row];
  end
  else
  begin
    Result := '';
  end;
end;


function TfrmNetworkDevices.GetLookupAddress : String;
begin
  Result := Self.CBInterfaces.Text;
end;


procedure TfrmNetworkDevices.SetLookupAddress(value : String);
begin
  Self.CBInterfaces.Text := value;
end;


procedure TfrmNetworkDevices.ResizeComponents;
//var offs : Integer;
//    newheight : Integer;
begin
{
  // Vertical Pos
  offs := 10;
  Self.BtnFind.Top := Self.ClientHeight - offs - Self.BtnFind.Height;
  Self.EditIpAddress.Top := Self.ClientHeight - offs - Self.EditIpAddress.Height;
  Self.BtnConnect.Top := Self.ClientHeight - offs - Self.BtnConnect.Height;
  Self.BtnClose.Top := Self.ClientHeight - offs - Self.BtnClose.Height;
  Self.LabelIPAddress.Top := Self.ClientHeight - offs - Self.EditIpAddress.Height - Self.LabelIPAddress.Height - offs;

  newheight := Self.LabelIPAddress.Top - offs - Self.GrpBoxDevices.Top;
  if (newheight < 20) then
  begin
    Self.ClientHeight := Self.ClientHeight + newheight;
  end
  else
  begin
    Self.GrpBoxDevices.Height := newheight;
  end;

  // Horizontal Pos
  offs := 10;
  Self.GrpBoxDevices.Width := Self.ClientWidth - 2 * Self.GrpBoxDevices.Left;
  Self.BtnClose.Left := Self.ClientWidth - offs - Self.BtnClose.Width;
  Self.BtnConnect.Left := Self.BtnClose.Left - offs - Self.BtnConnect.Width;
  }
end;


procedure TfrmNetworkDevices.ResizeColumns;
var i : Integer;
    ColWid : Integer;
begin
  ColWid := StringGridNetDevices.Width div StringGridNetDevices.ColCount;
  for i := 0 to StringGridNetDevices.ColCount - 1 do
  begin
    StringGridNetDevices.ColWidths[i] := ColWid;
  end;
end;


procedure TfrmNetworkDevices.FormResize(Sender: TObject);
begin
  ResizeComponents();
  ResizeColumns();
end;


procedure TfrmNetworkDevices.BtnCloseClick(Sender: TObject);
begin
  ModalResult := mrOk;
  Close();
end;


procedure TfrmNetworkDevices.InitiateDeviceFind(const ipaddress : String; const localbindip : String);
var buf: Array[1..8] of Char;
begin
  StringGridNetDevices.RowCount := 1;

  buf := 'BlahBlah';
  NMUDPSock.LocalHost := localbindip;
  NMUDPSock.RemoteHost := ipaddress;  // TODO some checks
  NMUDPSock.RemotePort := 2244;
  NMUDPSock.SendBuffer(buf, 8);
end;


procedure TfrmNetworkDevices.BtnFindClick(Sender: TObject);
var str : String;
    LocalIp : String;
    BcastStr : String;
    startpos : Integer;
begin
  str := Self.LookUpAddress;
  startpos := Pos(IpBCastSeparator, str);
  if (startpos > 0) then
  begin
    LocalIp := Copy(str, 0, startpos - 1);
    BcastStr := Copy(str, startpos + Length(IpBCastSeparator), Length(str) - startpos - Length(IpBCastSeparator) + 1);
  end
  else
  begin
    BcastStr := str;
    LocalIp := GetInterfaceForBroadcast(BcastStr);
  end;
  Self.InitiateDeviceFind(BcastStr, LocalIp);
end;


procedure TfrmNetworkDevices.NMUDPSockInvalidHost(var handled: Boolean);
begin
  MessageDlg('Invalid Hostname/IP Address: ' + CBInterfaces.Text, mtError, [mbOk], 0);
  handled := True;
end;


procedure TfrmNetworkDevices.NMUDPSockDataReceived(Sender: TComponent;
  NumberBytes: Integer; FromIP: String; Port: Integer);
var
  s2 : String;
  buf : Array[1..1024] of Char;
  len, i : Integer;
  rd : TRinstrumDevice;
begin
  NMUDPSock.ReadBuffer(buf, len);
  s2 := '';
  for i := 1 to len do
    s2 := s2 + buf[i];

  rd := TRinstrumDevice.Create();
  rd.ExtractFromXML(s2, FromIP);
  rd.AddToStringGrid(Self.StringGridNetDevices);
  rd.Free();
end;


procedure TfrmNetworkDevices.StringGridNetDevicesSelectCell(
  Sender: TObject; ACol, ARow: Integer; var CanSelect: Boolean);
begin
  if (CanConnect and (ARow > 0)) then
  begin
    BtnConnect.Enabled := True;
    BtnWeb.Enabled := True;
  end
  else
  begin
    BtnConnect.Enabled := False;
    BtnWeb.Enabled := False;
  end;
end;


procedure TfrmNetworkDevices.BtnWebClick(Sender: TObject);
var url : String;
begin
  url := 'http://' + Self.ConnectAddress + '/';
  ShellExecute(Handle, 'open', PChar(url), nil, nil, SW_SHOWNORMAL);
end;


procedure TfrmNetworkDevices.FormDestroy(Sender: TObject);
begin
  NMUDPSock.Free();
end;

end.
