unit comdialog;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, CPortCtl, adport, dmViewer, CPort;

type

  TFrmComDialog = class(TForm)
    BtnOk: TButton;
    BtnCancel: TButton;
    GBSerial: TGroupBox;
    RGConnection: TRadioGroup;
    RGParity: TRadioGroup;
    RGData: TRadioGroup;
    RGStop: TRadioGroup;
    GBTcp: TGroupBox;
    EdtIndHostname: TEdit;
    lblIndHostname: TLabel;
    EdtTcpPort: TEdit;
    lblTcpPort: TLabel;
    GBUnitAddress: TGroupBox;
    CBAddress: TComboBox;
    CBBaud: TComComboBox;
    lblBaud: TLabel;
    lblSerialPort: TLabel;
    CBSerialPort: TComboBox;
    procedure RGPhysicalClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure RGConnectionClick(Sender: TObject);
    procedure EdtTcpPortExit(Sender: TObject);
    procedure FormShow(Sender: TObject);
  private
    function GetConnectionType : TConnectionType;
    procedure SetConnectionType( ConnectionType : TConnectionType );
    function GetComPort : String;
    procedure SetComPort( Port : String );
    function GetBaudRate : Integer;
    procedure SetBaudRate( BaudRate : Integer );
    function GetStopBits : Integer;
    procedure SetStopBits( StopBits : Integer );
    function GetDataBits : Integer;
    procedure SetDataBits( DataBits : Integer );
    function GetParityBits : TParity;
    procedure SetParityBits( Parity : TParity );
    function GetSerialAddress : Integer;
    procedure SetSerialAddress( Address : Integer );
    function GetHostname : String;
    procedure SetHostname( Hostname : String );
    function GetTcpPort : Integer;
    procedure SetTcpPort( TcpPort : Integer );

    procedure ShowConnectionSettings( ConnectionType : TConnectionType );
  public
    property ConnectionType : TConnectionType read GetConnectionType write SetConnectionType;
    property ComPort : String read GetComPort write SetComPort;
    property BaudRate : Integer read GetBaudRate write SetBaudRate;
    property StopBits : Integer read GetStopBits write SetStopBits;
    property DataBits : Integer read GetDataBits write SetDataBits;
    property ParityBits : TParity read GetParityBits write SetParityBits;
    property SerialAddress : Integer read GetSerialAddress write SetSerialAddress;
    property Hostname : String read GetHostname write SetHostname;
    property TcpPort : Integer read GetTcpPort Write SetTcpPort;

    procedure LoadSettings( DatViewer : TDatViewer );
    procedure SaveSettings( DatViewer : TDatViewer );
  end;

var
  FrmComDialog: TFrmComDialog;

implementation

uses main;

{$R *.DFM}

procedure TFrmComDialog.RGPhysicalClick(Sender: TObject);
var Vis : Boolean;
begin
  Vis := True;

  CBBaud.Visible := Vis;
  RGParity.Visible := Vis;
  RGData.Visible := Vis;
  RGStop.Visible := Vis;
  GBUnitAddress.Visible := Vis;
end;


function TFrmComDialog.GetConnectionType : TConnectionType;
begin
  if ( RGConnection.ItemIndex = 0 ) then
    Result := ctSerial
  else
    Result := ctTcp;
end;


procedure TFrmComDialog.SetConnectionType( ConnectionType : TConnectionType );
begin
  if ( ConnectionType = ctSerial ) then
    RGConnection.ItemIndex := 0
  else
    RGConnection.ItemIndex := 1;
end;


function TFrmComDialog.GetComPort : String;
begin
  Result := CBSerialPort.Text;
end;


procedure TFrmComDialog.SetComPort( Port : String );
begin
  CBSerialPort.ItemIndex := CBSerialPort.Items.IndexOf( Port );
end;


function TFrmComDialog.GetBaudRate : Integer;
begin
  Result := StrToInt( CBBaud.Items[ CBBaud.ItemIndex ] );
end;


procedure TFrmComDialog.SetBaudRate( BaudRate : Integer );
begin
  CBBaud.ItemIndex := CBBaud.Items.IndexOf( IntToStr( BaudRate ) );
end;


function TFrmComDialog.GetStopBits : Integer;
begin
  Result := StrToInt( RGStop.Items[ RGStop.ItemIndex ] )
end;


procedure TFrmComDialog.SetStopBits( StopBits : Integer );
begin
  RGStop.ItemIndex := RGStop.Items.IndexOf( IntToStr( StopBits ) );
end;


function TFrmComDialog.GetDataBits : Integer;
begin
  Result := StrToInt( RGData.Items[ RGData.ItemIndex ] )
end;


procedure TFrmComDialog.SetDataBits( DataBits : Integer );
begin
  RGData.ItemIndex := RGData.Items.IndexOf( IntToStr( DataBits ) );
end;


function TFrmComDialog.GetParityBits : TParity;
begin
  Result := pNone;
  case ( RGParity.ItemIndex ) of
    0: Result := pNone;
    1: Result := pOdd;
    2: Result := pEven;
  end;
end;


procedure TFrmComDialog.SetParityBits( Parity : TParity );
begin
  RGParity.ItemIndex := 0;
  case ( Parity ) of
    pNone: RGParity.ItemIndex := 0;
    pOdd: RGParity.ItemIndex := 1;
    pEven: RGParity.ItemIndex := 2;
  end;
end;


function TFrmComDialog.GetSerialAddress : Integer;
begin
  if (CBAddress.ItemIndex = 0) then
  begin
    Result := 99;  // 99 Is "Any" address
  end
  else
  begin
    Result := CBAddress.ItemIndex;
  end
end;


procedure TFrmComDialog.SetSerialAddress( Address : Integer );
begin
  if (Address = 99) then
    CBAddress.ItemIndex := 0
  else
    CBAddress.ItemIndex := Address;
end;


function TFrmComDialog.GetHostname : String;
begin
  Result := EdtIndHostname.Text;
end;


procedure TFrmComDialog.SetHostname( Hostname : String );
begin
  EdtIndHostname.Text := Hostname;
end;


function TFrmComDialog.GetTcpPort : Integer;
begin
  Result := StrToInt( EdtTcpPort.Text );
end;


procedure TFrmComDialog.SetTcpPort( TcpPort : Integer );
begin
  EdtTcpPort.Text := IntToStr( TcpPort );
end;


procedure TFrmComDialog.ShowConnectionSettings( ConnectionType : TConnectionType );
begin
  GBSerial.Visible := ( ConnectionType = ctSerial );
  GBTcp.Visible := ( ConnectionType = ctTcp );
  GBUnitAddress.Visible := True;
end;


procedure TFrmComDialog.FormCreate(Sender: TObject);
begin
  CBSerialPort.Text := '';
  // Get list of serial ports
  try
    EnumComPorts(CBSerialPort.Items);
  except
  end;

  ConnectionType := DEFAULT_CONNECTION_TYPE;
  ComPort := DEFAULT_SERIAL_COM_PORT;
  BaudRate := DEFAULT_SERIAL_BAUD_RATE;
  DataBits := DEFAULT_SERIAL_DATA_BITS;
  StopBits := DEFAULT_SERIAL_STOP_BITS;
  ParityBits := DEFAULT_SERIAL_PARITY_BITS;
  SerialAddress := DEFAULT_SERIAL_ADDRESS;

  Hostname := DEFAULT_TCP_HOSTNAME;
  TcpPort := DEFAULT_TCP_PORT;

  GBSerial.Left := 8;
  GBTcp.Left := 8;
  Width := GBSerial.Width + 2 * GBSerial.Left + 8;
  ShowConnectionSettings( ConnectionType );
end;


procedure TFrmComDialog.RGConnectionClick(Sender: TObject);
begin
  ShowConnectionSettings( ConnectionType );
end;


procedure TFrmComDialog.EdtTcpPortExit(Sender: TObject);
begin
  try
    GetTcpPort();
  except
    EdtTcpPort.SetFocus();
    Raise;
  end;
end;


procedure TFrmComDialog.LoadSettings( DatViewer : TDatViewer );
begin
  ConnectionType := DatViewer.ConnectionType;
  ComPort := DatViewer.SerialComPort;
  BaudRate := DatViewer.SerialBaudRate;
  DataBits := DatViewer.SerialDataBits;
  StopBits := DatViewer.SerialStopBits;
  ParityBits := DatViewer.SerialParityBits;
  SerialAddress := DatViewer.SerialAddress;

  Hostname := DatViewer.TcpHostname;
  TcpPort := DatViewer.TcpPort;                        
end;


procedure TFrmComDialog.SaveSettings( DatViewer : TDatViewer );
begin
  DatViewer.ConnectionType := ConnectionType;
  DatViewer.SerialComPort := ComPort;
  DatViewer.SerialBaudRate := BaudRate;
  DatViewer.SerialDataBits := DataBits;
  DatViewer.SerialStopBits := StopBits;
  DatViewer.SerialParityBits := ParityBits;
  DatViewer.SerialAddress := SerialAddress;

  DatViewer.TcpHostname := Hostname;
  DatViewer.TcpPort := TcpPort;
end;

procedure TFrmComDialog.FormShow(Sender: TObject);
begin
  //frmMain.lang.WriteLabels(ChangeFileExt(Application.ExeName, '.lang'), self);
//  frmMain.lang.UpdateForm(self);
end;

end.
