unit database;

interface

uses
   classes, sysutils, comms, dialogs;

const
   NUM_LIN_PTS = 10;
   NUM_SETPOINTS = 8;
   NUM_MATERIALS = 2;
   NUM_STANDARD_COMPORTS = 4;
   NUM_ETHERNET_COMPORTS = 2;
   NUM_REMOTE_INPUTS = NUM_SETPOINTS;

   SETP_ACTIVE = 1;
   SETP_MAT1 = 2;
   SETP_MAT2 = 3;
   SETP_DUMP = 4;
   SETP_FINISH = 5;

   SETP_PAUSE =  8;  {TODO check this }

   ABS_MODE = 3;

   NUM_HEADER_LINES = 6;

type
Tsetpoint = record
            setp_type: integer;
            setp_src: integer;
            setp_dirn: integer;
            setp_logic: integer;
            setp_lock: integer;
            setp_alarm: integer;
           end;
Tmaterial = record
             name: string;
             delay: integer;
            end;
Tfeeder = record
             flight : integer;
             tol: integer;
             target: integer;
          end;

TCharSet = set of char;

TComPort1to4 = record
  use : integer;
  baud : integer;
  parity: integer;
  databits : integer;
  stopbits : integer;
  termEn : integer;
  dtrEn : integer;
  rs232En : integer;
  end;

TComPort5to6 = record
  ethPort : integer;
  end;

Tdata5100 = class
   private
     FCurrentViewerPort : Integer;

   public
     weighing_mode: integer;
     trade_mode: integer;
     dp_pos: integer;
     fs1: integer;
     fs2: integer;
     cal_weight: integer;
     res1, res2: integer;
     hi_res: boolean;
     addTare : integer;
     directZero, directSpan : integer;
     units: integer;
     av_setting, jitter, filter,motion, ztrac,zrange,zband: integer;
     zstartup: integer;
     lin_pc, lin_cor: array [0..NUM_LIN_PTS-1] of integer;
     id, serial,version : string;
     z_but, t_but,g_but,p_but,f_but: integer;

     rem : array[0..NUM_REMOTE_INPUTS] of integer; // 0=func button, 1-NUM_REMOTE_INPUTS = remote buttons

     setp_data: array [0.. NUM_SETPOINTS-1] of Tsetpoint;
     feeder_data : array [0.. NUM_SETPOINTS-1] of Tfeeder;

     ethDHCPEn : boolean;
     ethAddr : string;
     ethMask : string;
     ethGateway : string;
     ethDNS, ethDNS2 : string;
     ethMac : string;
     ethHostname : String;

     buzEn : boolean;

     netAddress: integer;

     portData14 : array [0..NUM_STANDARD_COMPORTS-1] of TComPort1to4;
     portData56 : array [0..NUM_ETHERNET_COMPORTS-1] of TComPort5to6;

     prhdr : array [0..NUM_HEADER_LINES-1] of string;
     pr_fmt, au_fmt: string;

     //ser2_mode,
     pr_mode, tic_col, tic_row, auto_type,auto_src : integer;

     procedure dbase_to_slist(list: Tstringlist);

     procedure slist_to_dbase(list: Tstringlist);
     procedure recipe_to_slist(all_recipes: boolean; list: Tstringlist);

     procedure get_settings(all_settings: boolean; cmd_list, reply_list: Tstringlist);
     procedure get_recipe(all_recipes: boolean; cmd_list, reply_list: Tstringlist);

     function cal_change(list: Tstringlist): boolean;
     procedure clear_setp(setp: integer; all: boolean);
     constructor create;
     destructor destroy; override;

     function Ethernet_GetFieldFromStr(s : string; id : integer; sep: char) : string;
     function Ethernet_GetStrFromFields(sa : array of string):string;

     function GetCurrentViewerPort: Integer;
     procedure SetCurrentViewerPort(port : Integer);

     property CurrentViewerPort: Integer read GetCurrentViewerPort write SetCurrentViewerPort;
  end;

implementation

uses
   tools;

const
 mADR = 95;
 mAFT = 149;
 mASF = 473;
 mBDR = 771;
 mBDX = 777;
 mBUZ = 1221;
 mCDL = 1441;
 mCLK = 1648;
 mCOF = 1721;
 mCWT = 1943;
 mDBG = 2060;
 mENU = 3062;
 mESR = 3189;
 mETH = 3205;
 mFPL = 3781;
 mIAD = 5411;
 mICR = 5477;
 mIDN = 5499;
 mLBT = 7481;
 mLDW = 7536;
 mLIC = 7646;
 mLIL = 7655;
 mLIS = 7662;
 mLIV = 7665;
 mLWT = 8027;
 mMTD = 8609;
 mMSV = 8601;
 mPFT = 10289;
 mPRS = 10600;
 mPRT = 10601;
 //mPRX = 10605;
 mPST = 10627;
 mRBT = 11537;
 mRES = 11614;
 mSTP = 12677;
 mTAR = 12861;
 mTAS = 12862;
 mTAV = 12865;
 mTDD = 12925;
 mTST = 13331;
 mVAL = 14207;
 mWFP = (((( (ord('W')-ord('A'))*26)+(ord('F')-ord('A')))*26)+(ord('P')-ord('A')));
 mWMD = 15187;
 mZST = 17387;





procedure Tdata5100.dbase_to_slist(list: Tstringlist);
var
  temp, i : integer;
begin
  list.clear();
  list.add(format('IDN"%s"',[id]));
  list.add(format('ADR%d',[netAddress]));

  if hi_res then temp := 1 else temp := 0;
  list.add(format('IAD1,%d,%d,%d,%d,%d',[fs1,dp_pos,res1+1,temp,addtare]));
  list.add(format('IAD2,%d,%d,%d,%d,%d',[fs2,dp_pos,res2+1,temp,addtare]));
  list.add(format('CWT%d',[cal_weight]));
  list.add(format('WMD%d,%d',[weighing_mode+1,trade_mode]));
  list.add(format('ENU%d',[units]));
  list.add(format('ICR%d',[filter]));
  list.add(format('ASF%d,%d',[av_setting,jitter]));
  list.add(format('LBT0,%d',[z_but]));
  list.add(format('LBT1,%d',[t_but]));
  list.add(format('LBT2,%d',[g_but]));
  list.add(format('LBT3,%d',[p_but]));
  list.add(format('LBT4,%d',[f_but]));

  list.add(format('MTD%d',[motion]));
  list.add(format('PFT"%s"',[pr_fmt]));
  list.add(format('AFT"%s"',[au_fmt]));
  for i := 0 to NUM_HEADER_LINES-1 do begin
    list.add(format('PST%d,"%s"',[i+1,prhdr[i]]));
//    ShowMEssage(format('PST%d,"%s"',[i+1,prhdr[i]]));
  end;
  list.add(format('PRS,%d,%d,%d,%d,%d',[pr_mode+1,auto_type+1, tic_col,tic_row,auto_src+1]));
  list.add(format('AFT"%s"',[au_fmt]));

  list.add(format('ZST%d,%d,%d,%d',[zstartup,ztrac,zrange+1,zband]));

  for i := 0 to NUM_REMOTE_INPUTS do
  begin
    list.add(format('RBT%d,%d', [i, rem[i]]));
  end;

  list.add(format('BUZ%d',[ord(buzEn)]));

  list.add(format('ETH%d,"%s","%s","%s","%s","%s","%s","%s"',[ord(ethDHCPEn),ethAddr,ethMask,ethGateway,ethDNS,ethDNS2,ethMac, ethHostname]));

  for i := 0 to NUM_STANDARD_COMPORTS - 1 do
  begin
    list.add(Format('BDX%d,%d,%d,%d,%d,%d,%d,%d,%d',
                    [i + 1,
                     portData14[i].use,
                     portData14[i].baud+1,
                     portData14[i].parity,
                     portData14[i].databits+7,
                     portData14[i].stopbits+1,
                     portData14[i].termEn,
                     portData14[i].dtrEn,
                     portData14[i].rs232En]));
  end;
  for i := 0 to NUM_ETHERNET_COMPORTS - 1 do
  begin
    list.add(format('BDX%d,,,,,,,,,%d', [i + NUM_STANDARD_COMPORTS + 1, portData56[i].ethPort]));
  end;

  for i := 0 to NUM_SETPOINTS-1 do begin
    list.add(format('LIV%d,%d,%d,%d,%d,%d,%d,%d,%d,%d',
      [i+1,setp_data[i].setp_type,
           setp_data[i].setp_src+1,
           setp_data[i].setp_dirn+1,
           feeder_data[i].target,
           feeder_data[i].flight,
           feeder_data[i].tol,
           setp_data[i].setp_logic+1,
           setp_data[i].setp_lock,
           setp_data[i].setp_alarm]));
  end;
end;



procedure Tdata5100.recipe_to_slist(all_recipes: boolean; list: Tstringlist);
begin
end;

procedure Tdata5100.get_settings(all_settings: boolean; cmd_list, reply_list: Tstringlist);
var
  i: integer;
begin
  cmd_list.clear();
  reply_list.clear();

  cmd_list.add('IDN?'); reply_list.add('IDN');
  cmd_list.add('ADR?'); reply_list.add('ADR');
  cmd_list.add('IAD?1'); reply_list.add('IAD');
  cmd_list.add('IAD?2'); reply_list.add('IAD');
  cmd_list.add('WMD?'); reply_list.add('WMD');
  cmd_list.add('ENU?'); reply_list.add('ENU');
  cmd_list.add('ICR?'); reply_list.add('ICR');
  cmd_list.add('LBT?0'); reply_list.add('LBT0,');
  cmd_list.add('LBT?1'); reply_list.add('LBT1,');
  cmd_list.add('LBT?2'); reply_list.add('LBT2,');
  cmd_list.add('LBT?3'); reply_list.add('LBT3,');
  cmd_list.add('LBT?4'); reply_list.add('LBT4,');
  cmd_list.add('MTD?'); reply_list.add('MTD');
  cmd_list.add('ZST?'); reply_list.add('ZST');
  cmd_list.add('CWT?'); reply_list.add('CWT');
  cmd_list.add('LDW?1'); reply_list.add('LDW');
  cmd_list.add('LWT?1'); reply_list.add('LWT');
  cmd_list.add('WFP?');  reply_list.add('WFP');
  if all_settings then begin
    cmd_list.add('ASF?'); reply_list.add('ASF');
    cmd_list.add('ETH?'); reply_list.add('ETH');
    cmd_list.add('BUZ?'); reply_list.add('BUZ');
    for i := 1 to NUM_STANDARD_COMPORTS + NUM_ETHERNET_COMPORTS do
    begin
      cmd_list.add(Format('BDX?%d', [i])); reply_list.add('BDX');
    end;
    cmd_list.add('PFT?'); reply_list.add('PFT');
    for i := 1 to NUM_HEADER_LINES do begin
      cmd_list.add('PST?' + inttostr(i)); reply_list.add('PST' + inttostr(i) + ',');
    end;
    cmd_list.add('PRS?'); reply_list.add('PRS');
    cmd_list.add('AFT?'); reply_list.add('AFT');

    for i := 0 to NUM_REMOTE_INPUTS do
    begin
      cmd_list.add(format('RBT?%d', [i])); reply_list.add(format('RBT%d,', [i]));
    end;

    for i := 1 to NUM_LIN_PTS do begin
      cmd_list.add(format('LIC?%d',[i])); reply_list.add(format('LIC%d,',[i]));
    end;

    for i:= 0 to NUM_SETPOINTS-1 do begin
      cmd_list.add(format('LIV?%d',[i+1])); reply_list.add('LIV');
    end;
  end;
end;

procedure Tdata5100.get_recipe(all_recipes: boolean; cmd_list, reply_list: Tstringlist);
begin
end;

procedure Tdata5100.slist_to_dbase(list: Tstringlist);
var
  i: integer;
  parsed_data: Tcommsobj;
  s: string;
  cmd: string;

function cmd_tag(): integer;
var
  tmp, i: integer;
begin
  tmp := 0;
  for i:= 1 to 3 do begin
    tmp := tmp * 26;
    tmp := tmp + ord(cmd[i]) - ord('A');
  end;
  result := tmp;
end;

procedure convert_data();
var
   temp, r,t,dp: integer;
begin
 with parsed_data do
  case cmd_tag() of
    mLDW: begin
            getIntParam(0, directZero, temp, 32000, -32000);
          end;
    mLWT: begin
            getIntParam(0, directSpan, temp, 32000, -32000);
          end;
    mIDN: begin
            getStrParam(0,id,10,0);
            getStrParam(1,serial,10,1);
            getStrParam(2,version,6,1);
          end;
    mAFT: getStrParam(0,au_fmt,20,0);
    mIAD: if getIntParam(0,temp,dp,2,1) then begin
            if temp = 1 then begin
               getIntParam(1,fs1,dp,999999,0);
               getIntParam(2,dp_pos,dp,5,0);
               if getIntParam(3,res1,dp,7,1) then dec(res1);
               getIntParam(4,temp,dp,1,0); hi_res := (temp = 1);
               getIntParam(5,addTare,dp,999999,0);
             end else begin
               getIntParam(1,fs2,dp,999999,0);
               if getIntParam(3,res2,dp,7,1) then dec(res2);
             end;
          end;

     mMTD: getIntParam(0,motion,dp,12,0);

     mASF: begin
             getIntParam(0,av_setting,dp,14,0);
             getintParam(1,jitter,dp,2,0);
           end;

     mENU: getIntParam(0,units,dp,4,0);


     mCWT: getIntParam(0,cal_weight,dp,999999,0);

     mICR: getIntParam(0,filter,dp,100,10);

     mADR: getIntParam(0,netAddress,dp,31,0);

     mWFP:
       begin
         getIntParam(0, temp, dp, 9999, 1);
         CurrentViewerPort := temp - 1;
       end;

     mBDX:
       begin
         // get port id
         if getIntParam(0, t, dp, 6, 1) then
         begin
           if (t > NUM_STANDARD_COMPORTS) then
           begin
             t := t - NUM_STANDARD_COMPORTS - 1; // get array index
             getIntParam(9, portData56[t].ethPort, dp, 65535, 0);
           end
           else
           begin
             t := t - 1; // get array index
             getIntParam(1, portData14[t].use,      dp, 7,  0);
             getIntParam(2, portData14[t].baud,     dp, 10, 1); portData14[t].baud := portData14[t].baud - 1;
             getIntParam(3, portData14[t].parity,   dp, 2,  0);
             getIntParam(4, portData14[t].databits, dp, 8,  7); portData14[t].databits := portData14[t].databits - 7;
             getIntParam(5, portData14[t].stopbits, dp, 2,  1); portData14[t].stopbits := portData14[t].stopbits - 1;
             getIntParam(6, portData14[t].termEn,   dp, 1,  0);
             getIntParam(7, portData14[t].dtrEn,    dp, 1,  0);
             getIntParam(8, portData14[t].rs232En,  dp, 1,  0);
           end;
         end;
       end;

     mBUZ:
       begin
         getIntParam(0,temp,dp,1,0); buzEn := boolean(temp);
       end;

     mETH:
       begin
         getIntParam(0,temp,dp,1,0); ethDHCPEn := boolean(temp);
         getStrParam(1,ethAddr,15,7);
         getStrParam(2,ethMask,15,7);
         getStrParam(3,ethGateway,15,7);
         getStrParam(4,ethDNS,15,7);
         getStrParam(5,ethDNS2,15,7);
         getStrParam(6,ethMac,23,7);
         getStrParam(7,ethHostname,64,0);
       end;

     mWMD:
       begin
         if getIntParam(0,weighing_mode,dp,4,1) then dec(weighing_mode);
         getIntParam(1,trade_mode,dp,1,0);
       end;

     mLBT:
       if (getIntParam(0,temp,dp,4,0)) then begin
         case temp of
           0: getIntParam(1,z_but,dp,2,0);
           1: getIntParam(1,t_but,dp,2,0);
           2: getIntParam(1,g_but,dp,2,0);
           3: getIntParam(1,p_but,dp,2,0);
           4: getIntParam(1,f_but,dp,2,0);
         end;
       end;

     mPFT: begin
             getStrParam(0,pr_fmt,100,0);
           end;
     mLIC: if (getIntParam(0, temp, dp, NUM_LIN_PTS, 1)) then
           begin
               dec(temp);
               if getintParam(1, r, dp, 150, -150) then
               begin
                   if getintParam(2, t, dp, 999999, -99999) then
                   begin
                       lin_pc[temp] := r;
                       lin_cor[temp] := t;
                   end;
               end;
           end;

     mPST: begin
             if (getIntParam(0,temp,dp,6,1)) then
               getStrParam(1,prhdr[temp-1],30,0);
           end;

     mPRS: begin
            //getIntParam(0,ser2_mode,dp,3,0);
            if getIntParam(1,pr_mode,dp,8,1) then dec(pr_mode);
            if getIntParam(2,auto_type,dp,6,1) then dec(auto_type);
            getIntParam(3,tic_col,dp,20,0);
            getIntParam(4,tic_row,dp,10,0);
            if getIntParam(5,auto_src,dp,4,1) then dec(auto_src);
           end;
       
      mZST: begin
              getIntParam(0,zstartup,dp,1,0);
              getIntParam(1,ztrac,dp,12,0);
              if getIntParam(2,zrange,dp,4,1) then dec(zrange);
              getIntParam(3,zband,dp,999999,0);
            end;
      mRBT: if (getIntParam(0, temp, dp, NUM_REMOTE_INPUTS, 0)) then
            begin
              getIntParam(1, rem[temp], dp, 21, 0);
            end;
       mLIV:
         if (getIntParam(0,r,dp,NUM_SETPOINTS, 1)) then begin
           r := r - 1;
           getIntParam(1,setp_data[r].setp_type,dp,50,0);
           if getIntParam(2,setp_data[r].setp_src,dp,2,1) then dec(setp_data[r].setp_src);
           if getIntParam(3,setp_data[r].setp_dirn,dp,2,1) then dec(setp_data[r].setp_dirn);
           getIntParam(4,feeder_data[r].target,dp,999999,-99999);
           getIntParam(5,feeder_data[r].flight,dp,999999,0);
           getIntParam(6,feeder_data[r].tol,dp,999999,0);
           if getIntParam(7,setp_data[r].setp_logic,dp,2,1) then dec(setp_data[r].setp_logic);
           getIntParam(8,setp_data[r].setp_lock,dp,1,0);
           getIntParam(9,setp_data[r].setp_alarm,dp,3,0);
         end;
    end;

end;


begin
  parsed_data := Tcommsobj.create();
  try
    for i := 0 to list.count-1 do begin
       s := list.strings[i];
       cmd := s;
       setlength(cmd,3);
       parsed_data.ParseStr(s,FALSE,TRUE);
       convert_data();
    end;
  finally
    parsed_data.destroy();
  end;
end;


destructor Tdata5100.destroy;
begin
end;


constructor Tdata5100.create;
var
 i: integer;

begin
   dp_pos := 0;
   weighing_mode := 0;
   trade_mode := 0;

   dp_pos := 0;
   fs1 := 4000;
   fs2 := 6000;
   cal_weight:= 3000;
   res1 := 0; res2 := 1;
   hi_res:= false;
   addTare := 0;
   directZero := 0;
   directSpan := 20000;
   units:= 2;
   av_setting := 0;
   jitter := 0;
   filter := 50;
   motion := 1;
   ztrac := 0;
   zrange := 3;
   zband := 0;
   zstartup := 0;
   for i := 0 to NUM_LIN_PTS-1 do
   begin
     lin_pc[i] := 0;
     lin_cor[i] := 0;
   end;

   id := '';
   serial := '';
   version := '';
   for i := 0 to NUM_HEADER_LINES-1 do prhdr[i] := '';
   pr_fmt := '';
   au_fmt := '';

   //ser2_mode := 0;
   pr_mode := 0;
   auto_type := 0;
   auto_src := 0;
   tic_col := 0;
   tic_row := 0;
   
   z_but := 1; t_but := 1; g_but := 1; p_but := 1; f_but := 1;

   for i := 0 to NUM_REMOTE_INPUTS do
     rem[i] := 0;

   for i := 0 to NUM_SETPOINTS-1 do begin
     setp_data[i].setp_type := 0;
     setp_data[i].setp_src := 0;
     setp_data[i].setp_dirn := 0;
     setp_data[i].setp_logic := 0;
     setp_data[i].setp_lock := 0;
     setp_data[i].setp_alarm := 0;

     feeder_data[i].flight := 0;
     feeder_data[i].tol := 0;
     feeder_data[i].target := 0;
   end;
   
   ethDHCPEn := TRUE;
   ethAddr := '0.0.0.0';
   ethMask := '255.255.255.255';
   ethGateway := '0.0.0.0';
   ethDNS := '0.0.0.0';
   ethDNS2 := '0.0.0.0';
   ethMac := '0.0.0.0.0.0';
   ethHostname := '';
   for i := 0 to NUM_STANDARD_COMPORTS - 1 do
   begin
     portData14[i].use := 0;
     portData14[i].baud := 0;
     portData14[i].parity := 0;
     portData14[i].databits := 0;
     portData14[i].stopbits := 0;
     portData14[i].termEn := 0;
     portData14[i].dtrEn := 0;
   end;
   for i := 0 to NUM_ETHERNET_COMPORTS - 1 do
   begin
     portData56[i].ethPort := 2222 + i;
   end;
   CurrentViewerPort := 0;
end;


procedure TData5100.clear_setp(setp: integer; all: boolean);
var
   first, last, i: integer;
begin
  if all then begin
    first := 0;
    last := NUM_SETPOINTS-1;
  end else begin
     first := setp;
     last := setp;
  end;

  for i := first to last do begin
     setp_data[i].setp_type := 0;
     setp_data[i].setp_src := 0;
     setp_data[i].setp_dirn := 0;
     setp_data[i].setp_logic := 0;
     setp_data[i].setp_lock := 0;

     feeder_data[i].flight := 0;
     feeder_data[i].tol := 0;
  end;
end;


function Tdata5100.cal_change(list: Tstringlist): boolean;
var
  cmd: string;
  i: integer;

function cmd_tag(): integer;
var
  tmp, i: integer;
begin
  tmp := 0;
  for i:= 1 to 3 do begin
    tmp := tmp * 26;
    tmp := tmp + ord(cmd[i]) - ord('A');
  end;
  result := tmp;
end;

begin
  result := false;
  for i := 0 to list.count-1 do begin
     cmd := list.strings[i];
     case cmd_tag() of
        mIAD,mWMD,mENU,mICR,mLBT,mMTD,mZST,mCWT: begin
             result := true;
             exit;
             end;
      end;
   end;
end;

function Tdata5100.Ethernet_GetFieldFromStr(s : string; id : integer; sep: char) : string;
var
  y, i : integer;
begin
  if id < 0 then id := 0;
  result := '';

  i := 0;
  while length(s) > 0 do begin
    // find start and end of the next section
    y := Pos(sep,s);
    if y = 0 then y := length(s)+1;

    if i = id then begin
      // if we need this string
      Delete(s,y,100);
      result := s;
      exit;
    end else begin
      // if we need a future string
      Delete(s,1,y);
    end;
    i := i + 1;
  end;
end;

procedure RemoveInvalidChars(var s:string; valid: TCharSet);
var
  i : integer;
begin
  i := 1;
  while i <= length(s) do begin
    if not (s[i] in valid) then Delete(s,i,1);
    i := i + 1;
  end;
end;

function Tdata5100.Ethernet_GetStrFromFields(sa : array of string):string;
var
  i : integer;
begin
  // get rid of invalid chars and make sure that
  for i := 0 to length(sa)-1 do begin
    RemoveInvalidChars(sa[i],['0'..'9']);
    if sa[i] = '' then begin
      result := '';
      exit;
    end;
  end;

  result := sa[0];
  for i := 1 to length(sa)-1 do begin
    result := result + '.' +  sa[i];
  end;
end;

function Tdata5100.GetCurrentViewerPort: Integer;
begin
  Result := FCurrentViewerPort;
end;

procedure Tdata5100.SetCurrentViewerPort(port : Integer);
begin
  // WFP is broken for TCP ports (6 and above), so we limit it here
  if (port > 6) then
    port := 6;
  FCurrentViewerPort := port;
end;

end.
