unit dmViewer;

interface

uses
  Forms, SysUtils, Classes, inifiles, adport;

type
  TRinCable = ( rcRS232, rcOptical );
  TConnectionType = ( ctSerial, ctTcp );

const
  DEFAULT_CONNECTION_TYPE = ctSerial;
  DEFAULT_SERIAL_COM_PORT = 'COM1';
  DEFAULT_SERIAL_BAUD_RATE = 9600;
  DEFAULT_SERIAL_DATA_BITS = 8;
  DEFAULT_SERIAL_STOP_BITS = 1;
  DEFAULT_SERIAL_PARITY_BITS = pNone;
  DEFAULT_SERIAL_CABLE = rcRS232;
  DEFAULT_SERIAL_ADDRESS = 0;
  DEFAULT_TCP_HOSTNAME = '192.168.1.70';
  DEFAULT_TCP_PORT = 2222;
  DEFAULT_DEVICE_FINDER_ADDRESS = '192.168.1.255';
  DEFAULT_LANG = 1;
  DEFAULT_DP_CHAR = '.';
  DEFAULT_COMMS_LOGGING_ENABLED = False;
  DEFAULT_COMMS_LOGGING_FILE = 'C:\C510Viewer.log';
  DEFAULT_TCP_TIMEOUT = 70;
  DEFAULT_SER_TIMEOUT = 20;

type

  TdatViewer = class(TDataModule)
    procedure DataModuleCreate(Sender: TObject);
    procedure DataModuleDestroy(Sender: TObject);
  private
    { Private declarations }
    FiniFile: TMemIniFile;
  public
    { Public declarations }

  protected

    // Connection settings
    procedure SetConnectionType( ConnType : TConnectionType );
    function GetConnectionType : TConnectionType;

    // Serial settings
    procedure SetSerialComPort( Port : String );
    function GetSerialComPort : String;
    procedure SetSerialBaudRate( BaudRate : Integer );
    function GetSerialBaudRate : Integer;
    procedure SetSerialDataBits( DataBits : Integer );
    function GetSerialDataBits : Integer;
    procedure SetSerialStopBits( StopBits : Integer );
    function GetSerialStopBits : Integer;
    procedure SetSerialParityBits( Parity : TParity );
    function GetSerialParityBits : TParity;
    procedure SetSerialAddress( Address : Integer );
    function GetSerialAddress : Integer;
    procedure SetSerialCable( Cable : TRinCable );
    function GetSerialCable : TRinCable;

    // TCP settings
    procedure SetTcpHostname( Hostname : String );
    function GetTcpHostname : String;
    procedure SetTcpPort( Port : Integer );
    function GetTcpPort : Integer;

    // Device finder
    procedure SetDeviceFinderLookupAddress( IpAddress : String );
    function GetDeviceFinderLookupAddress : String;

    // Decimal point character
    procedure SetDecimalPointChar( dpchar : Char );
    function GetDecimalPointChar : Char;

    // Comms protocol logging
    function GetCommsLoggingEnabled : Boolean;
    function GetCommsLoggingFile : String;

    // Timeout settings
    procedure SetTcpTimeout( timeout : Integer );
    function GetTcpTimeout : Integer;
    procedure SetSerTimeout( timeout : Integer );
    function GetSerTimeout : Integer;
    function GetTimeout : Integer;

    // First run flag
    procedure SetFirstRun(onOff : boolean);
    function GetFirstRun : boolean;

    // Display background image
    function GetBackgroundImageFilename : String;

  public
    property ConnectionType : TConnectionType read GetConnectionType write SetConnectionType;
    property SerialComPort : String read GetSerialComPort write SetSerialComPort;
    property SerialBaudRate : Integer read GetSerialBaudRate write SetSerialBaudRate;
    property SerialDataBits : Integer read GetSerialDataBits write SetSerialDataBits;
    property SerialStopBits : Integer read GetSerialStopBits write SetSerialStopBits;
    property SerialParityBits : TParity read GetSerialParityBits write SetSerialParityBits;
    property SerialAddress : Integer read GetSerialAddress write SetSerialAddress;
    property SerialCable : TRinCable read GetSerialCable write SetSerialCable;
    property TcpHostname : String read GetTcpHostname write SetTcpHostname;
    property TcpPort : Integer read GetTcpPort write SetTcpPort;
    property DeviceFinderLookupAddress : String read GetDeviceFinderLookupAddress write SetDeviceFinderLookupAddress;
    property DecimalPointChar : Char read GetDecimalPointChar write SetDecimalPointChar;
    property CommsLoggingEnabled : Boolean read GetCommsLoggingEnabled;
    property CommsLoggingFile : String read GetCommsLoggingFile;
    property TcpTimeout : integer read GetTcpTimeout write SetTcpTimeout;
    property SerTimeout : integer read GetSerTimeout write SetSerTimeout;
    property Timeout : integer read GetTimeout;
    property firstRun : boolean read GetFirstRun write SetFirstRun;
    property BackgroundImageFilename : String read GetBackgroundImageFilename;
  end;

var
  datViewer : TdatViewer;

implementation


{$R *.dfm}

procedure TdatViewer.DataModuleCreate(Sender: TObject);
begin
  // load the inifile & test
  if FileExists(ChangeFileExt(Application.ExeName, '.ini')) then
  begin
    FiniFile := TMemIniFile.Create(ChangeFileExt(Application.ExeName, '.ini'));
  end;
end;

procedure TdatViewer.DataModuleDestroy(Sender: TObject);
begin
  if Assigned(FiniFile) then
  begin
    FiniFile.UpdateFile;
    FiniFile.Free;
    FiniFile := nil;
  end;
end;


// Connection settings


procedure TdatViewer.SetConnectionType( ConnType : TConnectionType );
begin
  if Assigned( FiniFile ) then
  begin
    FiniFile.WriteInteger( 'Connection', 'ConnectionType', Integer( ConnType ) );
  end;
end;


function TdatViewer.GetConnectionType : TConnectionType;
begin
  Result := DEFAULT_CONNECTION_TYPE;
  if Assigned( FiniFile ) then
  begin
    Result := TConnectionType( FiniFile.ReadInteger( 'Connection', 'ConnectionType', Integer( DEFAULT_CONNECTION_TYPE ) ) );
  end;
end;


// Serial settings


procedure TdatViewer.SetSerialComPort( Port : String );
begin
  if Assigned( FiniFile ) then
  begin
    FiniFile.WriteString( 'Connection', 'SerialPort', Port );
  end;
end;


function TdatViewer.GetSerialComPort : String;
begin
  Result := DEFAULT_SERIAL_COM_PORT;
  if Assigned( FiniFile ) then
  begin
    Result := FiniFile.ReadString( 'Connection', 'SerialPort', DEFAULT_SERIAL_COM_PORT );
  end;
end;


procedure TdatViewer.SetSerialBaudRate( BaudRate : Integer );
begin
  if Assigned( FiniFile ) then
  begin
    FiniFile.WriteInteger( 'Connection', 'BaudRate', BaudRate );
  end;
end;


function TdatViewer.GetSerialBaudRate : Integer;
begin
  Result := DEFAULT_SERIAL_BAUD_RATE;
  if Assigned( FiniFile ) then
  begin
    Result := FiniFile.ReadInteger( 'Connection', 'BaudRate', DEFAULT_SERIAL_BAUD_RATE );
  end;
end;


procedure TdatViewer.SetSerialDataBits( DataBits : Integer );
begin
  if Assigned( FiniFile ) then
  begin
    FiniFile.WriteInteger( 'Connection', 'DataBits', DataBits );
  end;
end;


function TdatViewer.GetSerialDataBits : Integer;
begin
  Result := DEFAULT_SERIAL_DATA_BITS;
  if Assigned( FiniFile ) then
  begin
    Result := FiniFile.ReadInteger( 'Connection', 'DataBits', DEFAULT_SERIAL_DATA_BITS );
  end;
end;


procedure TdatViewer.SetSerialStopBits( StopBits : Integer );
begin
  if Assigned( FiniFile ) then
  begin
    FiniFile.WriteInteger( 'Connection', 'StopBits', StopBits );
  end;
end;


function TdatViewer.GetSerialStopBits : Integer;
begin
  Result := DEFAULT_SERIAL_STOP_BITS;
  if Assigned( FiniFile ) then
  begin
    Result := FiniFile.ReadInteger( 'Connection', 'StopBits', DEFAULT_SERIAL_STOP_BITS );
  end;
end;


procedure TdatViewer.SetSerialParityBits( Parity : TParity );
begin
  if Assigned( FiniFile ) then
  begin
    FiniFile.WriteInteger( 'Connection', 'ParityBits', Integer( Parity ) );
  end;
end;


function TdatViewer.GetSerialParityBits : TParity;
begin
  Result := DEFAULT_SERIAL_PARITY_BITS;
  if Assigned( FiniFile ) then
  begin
    Result := TParity( FiniFile.ReadInteger( 'Connection', 'ParityBits', Integer( DEFAULT_SERIAL_PARITY_BITS ) ) );
  end;
end;


procedure TdatViewer.SetSerialAddress( Address : Integer );
begin
  if Assigned( FiniFile ) then
  begin
    FiniFile.WriteInteger( 'Connection', 'SerialAddress', Address );
  end;
end;


function TdatViewer.GetSerialAddress : Integer;
begin
  Result := DEFAULT_SERIAL_ADDRESS;
  if Assigned( FiniFile ) then
  begin
    Result := FiniFile.ReadInteger( 'Connection', 'SerialAddress', DEFAULT_SERIAL_ADDRESS );
  end;
end;


procedure TdatViewer.SetSerialCable( Cable : TRinCable );
begin
  if Assigned( FiniFile ) then
  begin
    FiniFile.WriteInteger( 'Connection', 'SerialCable', Integer( Cable ) );
  end;
end;


function TdatViewer.GetSerialCable : TRinCable;
begin
  Result := DEFAULT_SERIAL_CABLE;
  if Assigned( FiniFile ) then
  begin
    Result := TRinCable( FiniFile.ReadInteger( 'Connection', 'SerialCable', Integer( DEFAULT_SERIAL_CABLE ) ) );
  end;
end;


// TCP settings


procedure TdatViewer.SetTcpHostname( Hostname : String );
begin
  if Assigned( FiniFile ) then
  begin
    FiniFile.WriteString( 'Connection', 'TcpHostname', Hostname );
  end;
end;


function TdatViewer.GetTcpHostname : String;
begin
  Result := DEFAULT_TCP_HOSTNAME;
  if Assigned( FiniFile ) then
  begin
    Result := FiniFile.ReadString( 'Connection', 'TcpHostname', DEFAULT_TCP_HOSTNAME );
  end;
end;


procedure TdatViewer.SetTcpPort( Port : Integer );
begin
  if Assigned( FiniFile ) then
  begin
    FiniFile.WriteInteger( 'Connection', 'TcpPort', Port );
  end;
end;


function TdatViewer.GetTcpPort : Integer;
begin
  Result := DEFAULT_TCP_PORT;
  if Assigned( FiniFile ) then
  begin
    Result := FiniFile.ReadInteger( 'Connection', 'TcpPort', DEFAULT_TCP_PORT );
  end;
end;


procedure TdatViewer.SetDeviceFinderLookupAddress( IpAddress : String );
begin
  if Assigned( FiniFile ) then
  begin
    FiniFile.WriteString( 'DeviceFinder', 'Address', IpAddress );
  end;
end;


function TdatViewer.GetDeviceFinderLookupAddress : String;
begin
  Result := DEFAULT_DEVICE_FINDER_ADDRESS;
  if Assigned( FiniFile ) then
  begin
    Result := FiniFile.ReadString( 'DeviceFinder', 'Address', DEFAULT_DEVICE_FINDER_ADDRESS );
  end;
end;


procedure TdatViewer.SetDecimalPointChar( dpchar : Char );
begin
  if Assigned( FiniFile ) then
  begin
    FiniFile.WriteString( 'General', 'DecimalPointChar', dpchar );
  end;
end;


function TdatViewer.GetDecimalPointChar : Char;
var str : String;
begin
  Result := DEFAULT_DP_CHAR;
  if Assigned( FiniFile ) then
  begin
    str := FiniFile.ReadString( 'General', 'DecimalPointChar', DEFAULT_DP_CHAR );
    if (Length(str) >= 1) then
        Result := str[1];
  end;
end;


function TdatViewer.GetCommsLoggingEnabled : Boolean;
begin
  Result := DEFAULT_COMMS_LOGGING_ENABLED;
  if Assigned(FiniFile) then
  begin
    Result := FiniFile.ReadBool('General', 'CommsLoggingEnabled', DEFAULT_COMMS_LOGGING_ENABLED);
  end;
end;


function TdatViewer.GetCommsLoggingFile : String;
begin
  Result := DEFAULT_COMMS_LOGGING_FILE;
  if Assigned(FiniFile) then
  begin
    Result := FiniFile.ReadString('General', 'CommsLoggingFile', DEFAULT_COMMS_LOGGING_FILE);
  end;
end;


procedure TdatViewer.SetTcpTimeout( timeout : Integer );
begin
  if Assigned( FiniFile ) then
  begin
    FiniFile.WriteInteger( 'Connection', 'TcpTimeout', timeout );
  end;
end;


function TdatViewer.GetTcpTimeout : Integer;
begin
  Result := DEFAULT_TCP_TIMEOUT;
  if Assigned( FiniFile ) then
  begin
    Result := FiniFile.ReadInteger( 'Connection', 'TcpTimeout', DEFAULT_TCP_TIMEOUT );
  end;
end;


procedure TdatViewer.SetSerTimeout( timeout : Integer );
begin
  if Assigned( FiniFile ) then
  begin
    FiniFile.WriteInteger( 'Connection', 'SerTimeout', timeout );
  end;
end;


function TdatViewer.GetSerTimeout : Integer;
begin
  Result := DEFAULT_SER_TIMEOUT;
  if Assigned( FiniFile ) then
  begin
    Result := FiniFile.ReadInteger( 'Connection', 'SerTimeout', DEFAULT_SER_TIMEOUT );
  end;
end;


function TdatViewer.GetTimeout : Integer;
begin
  if ConnectionType = ctTcp then begin
    result := GetTcpTimeout;
  end else begin
    result := GetSerTimeout;
  end;
end;


procedure TdatViewer.SetFirstRun( onOff : boolean );
begin
  if Assigned( FiniFile ) then
  begin
    FiniFile.WriteBool( 'Options', 'FirstRun', onOff );
  end;
end;


function TdatViewer.GetFirstRun : boolean;
begin
  Result := True;
  if Assigned( FiniFile ) then
  begin
    Result := FiniFile.ReadBool( 'Options', 'FirstRun', result );
  end;
end;


function TdatViewer.GetBackgroundImageFilename : String;
begin
  Result := '';
  if Assigned(FiniFile) then
  begin
    Result := FiniFile.ReadString('General', 'BackgroundImage', '');
  end;
end;


end.

