unit IOIfaceConfigure;

interface

uses comdialog, ioiface, controls, dmViewer;


function IsIpAddress( Str : String ) : Boolean;
function CreateConfigureIOInterface( DatViewer : TDatViewer; var Address : Integer ) : TIOIface;


implementation

uses sysutils;


function IsIpAddress( Str : String ) : Boolean;
var
  DotPosn : Integer;
  TempStr : String;
  TempInt : Integer;
  i : Integer;
begin
  Result := true;
  try
    for i := 0 to 3 do
    begin
      DotPosn := Pos( '.', Str );
      if (DotPosn < 0) then
      begin
        Result := false;
        exit;
      end;
      TempStr := Copy( Str, 0, DotPosn - 1 );
      Delete( Str, 0, DotPosn );
      TempInt := StrToInt( TempStr );
      if ( TempInt > 255 ) then
      begin
        Result := false;
        exit;
      end;
    end;
  except
    Result := false;
  end;
end;


function CreateConfigureIOInterface( DatViewer : TDatViewer; var Address : Integer ) : TIOIface;
var FrmComDialog : TFrmComDialog;
    IOIfaceSerial : TIOIfaceSerial;
    IOIfaceSocket : TIOIfaceSocket;
    IOIface : TIOIface;
begin
  IOIface := nil;
  FrmComDialog := TFrmComDialog.Create( nil );
  try
    // Get setting from ini file
    FrmComDialog.LoadSettings( DatViewer );
    if ( FrmComDialog.ShowModal() = mrOk ) then
    begin

      // Save settings to ini file
      FrmComDialog.SaveSettings( DatViewer );
      Address := FrmComDialog.SerialAddress;

      if ( FrmComDialog.ConnectionType = ctSerial ) then
      begin
        // Configure serial interface
        IOIfaceSerial := TIOIfaceSerial.Create( nil );
        IOIfaceSerial.ComPort := FrmComDialog.ComPort;
        IOIfaceSerial.BaudRate := FrmComDialog.BaudRate;
        IOIfaceSerial.DataBits := FrmComDialog.DataBits;
        IOIfaceSerial.StopBits := FrmComDialog.StopBits;
        IOIfaceSerial.ParityBits := FrmComDialog.ParityBits;
        IOIface := IOIfaceSerial;
      end
      else
      begin
        // Configure TCP interface
        IOIfaceSocket := TIOIfaceSocket.Create( nil );
        if ( IsIpAddress( FrmComDialog.Hostname ) ) then
          IOIfaceSocket.Address := FrmComDialog.Hostname
        else
          IOIfaceSocket.Host := FrmComDialog.Hostname;
        IOIfaceSocket.Port := FrmComDialog.TcpPort;
        IOIface := IOIfaceSocket;
      end;
    end;
  finally
    FrmComDialog.Free();
  end;
  Result := IOIface;
end;


end.
