unit database;

interface

uses
   classes, sysutils, comms, dialogs, indicator_model;

type
   //          0         1         2         3         4          5           6            7
   TBDXPort = (BDXMod1A, BDXMod1B, BDXMod2A, BDXMod2B, BDXTcpNet, BDXTcpAuto, BDXUsbSlave, BDXUsbHost,
               BDXUnused1, BDXUnused2, BDXSerial1, BDXUdpO1, BDXUdpO2, BDXUdpI1, BDXUdpI2, BDXUnused3, BDXUnused4, BDXSerial2);
   //          8           9           10          11        12        13        14        15          16          17

const
   NUM_LIN_PTS = 10;
   NUM_SETPOINTS = 8;
   NUM_MATERIALS = 2;
   NUM_REMOTE_INPUTS = NUM_SETPOINTS;
   NUM_FRONT_FUNC = 3;

   SETP_TYPE_OFF = 0;
   SETP_TYPE_ON = 1;
   SETP_TYPE_OVER = 2;
   SETP_TYPE_UNDER = 3;
   SETP_TYPE_COZ = 4;
   SETP_TYPE_ZERO = 5;
   SETP_TYPE_NET = 6;
   SETP_TYPE_MOTION = 7;
   SETP_TYPE_ERROR = 8;
   SETP_TYPE_BUZZER = 9;
   SETP_TYPE_WEIGHOUT = 10;
   SETP_TYPE_WEIGHIN = 11;

   NUM_HEADER_LINES = 6;

   BDXPortNames : Array[TBDXPort] of String =
     ( 'SER.M.1A', 'SER.M.1B', 'SER.M.2A', 'SER.M.2B',
       'TCP.IN.1', 'TCP.IN.2', 'SER.SLV',  'SER.HST',
       'Unused',   'Unused',   'SER.1',    'UDP.O.1',
       'UDP.O.2',  'UDP.IN.1', 'UDP.IN.2', 'Unused',
       'Unused',   'SER.2' );

type
Tsetpoint = record
            setp_type: integer;
            setp_src: integer;
            setp_logic: integer;
            setp_lock: integer;
            setp_alarm: integer;
            setp_timing: integer;
            setp_reset: integer;
            setp_inflight : integer;
            setp_hysteresis: integer;
            setp_target: integer;
           end;

TCharSet = set of char;

TComPortStandard = record
  use : integer;
  baud : integer;
  parity: integer;
  databits : integer;
  stopbits : integer;
  termEn : integer;
  dtrEn : integer;
  rs232En : integer;
  tcpPort : integer;
  tcpTimeout : integer;
  udpDestIP : string;
end;

TComPortTCP = record
  tcpPort : integer;
end;

Tdata5100 = class
   private
     FCurrentViewerPort : Integer;

   public
     weighing_mode: integer;
     trade_mode: integer;
     dp_pos: integer;
     fs1: integer;
     fs2: integer;
     cal_weight: integer;
     senseLine: integer;
     res1, res2: integer;
     hi_res: boolean;
     addTare : integer;
     directZero, directSpan : integer;
     units: integer;
     av_setting, jitter, filter,motion, ztrac,zrange,zband: integer;
     zstartup: integer;
     lin_pc, lin_cor: array [0..NUM_LIN_PTS-1] of integer;
     id, serial, version, model : String;
     model_subtype : Integer;
     z_but, t_but, g_but : Integer;
     f_but: array[1..NUM_FRONT_FUNC] of Integer;
     secondary_units : Integer;
     tertiary_units : Integer;
     conversion_factor : Integer;
     show_lboz : Integer;

     rem : array[1..NUM_REMOTE_INPUTS] of integer; // 1-NUM_REMOTE_INPUTS = remote buttons
     front_func : array[1..NUM_FRONT_FUNC] of Integer;

     setp_data: array [0.. NUM_SETPOINTS-1] of Tsetpoint;
     setp_opt_jog_on : Integer;
     setp_opt_jog_off : Integer;
     setp_opt_jogs_in_set : Integer;
     setp_opt_max_jog_sets : Integer;
     setp_opt_feeder_type : Integer;
     setp_opt_delay_check : Integer;

     ethDHCPEn : boolean;
     ethAddr : string;
     ethMask : string;
     ethGateway : string;
     ethDNS1 : string;
     ethDNS2 : string;
     ethDNS3 : string;
     ethMac : string;
     ethHostname : String;
     ethSearchDomain : String;

     buzEn : boolean;

     netAddress: integer;

     portData14 : array[TBDXPort] of TComPortStandard;

     prhdr : array [0..NUM_HEADER_LINES-1] of string;
     pr_fmt, au_fmt: string;

     //ser2_mode,
     pr_mode, tic_col, tic_row, auto_type, auto_src : integer;
     print_interlock : Integer;
     auto_start_char : Integer;
     auto_end_char1 : Integer;
     auto_end_char2 : Integer;

     mbPort : integer;
     mbEndian : integer;

     anlType, anlSource, anlZero, anlSpan, anlFrc, anlAbs, anlClip : integer;

     clkHour, clkMinute, clkSecond, clkDay, clkMonth, clkYear : integer;
     time_format : Integer;
     date_format : Integer;

     qafEnable, qafYear, qafMonth, qafDay : integer;

     brightness : integer;

     procedure dbase_to_slist(list: Tstringlist);

     procedure slist_to_dbase(list: Tstringlist);
     procedure recipe_to_slist(all_recipes: boolean; list: Tstringlist);

     procedure get_settings(all_settings: boolean; cmd_list, reply_list: Tstringlist);
     procedure get_recipe(all_recipes: boolean; cmd_list, reply_list: Tstringlist);

     function cal_change(list: Tstringlist): boolean;
     procedure clear_setp(setp: integer; all: boolean);
     constructor create;
     destructor destroy; override;

     function Ethernet_GetFieldFromStr(s : string; id : integer; sep: char) : string;
     procedure Ethernet_GetIpOctetsFromStr(s : String; sep: Char; var oct0, oct1, oct2, oct3 : Integer);
     function Ethernet_GetStrFromFields(sa : array of string):string;

     function GetCurrentViewerPort: Integer;
     procedure SetCurrentViewerPort(port : Integer);

     property CurrentViewerPort: Integer read GetCurrentViewerPort write SetCurrentViewerPort;
  end;

implementation

uses
   tools;

const
 mADR = 95;
 mAFT = 149;
 mAOC = (Ord('A') - Ord('A')) * 26 * 26 + (Ord('O') - Ord('A')) * 26 + (Ord('C') - Ord('A'));
 mASF = 473;
 mBDR = 771;
 mBDX = 777;
 mBUZ = 1221;
 mBRT = (Ord('B') - Ord('A')) * 26 * 26 + (Ord('R') - Ord('A')) * 26 + (Ord('T') - Ord('A'));
 mCDL = 1441;
 mCFM = (Ord('C') - Ord('A')) * 26 * 26 + (Ord('F') - Ord('A')) * 26 + (Ord('M') - Ord('A'));
 mCLK = 1648;
 mCOF = 1721;
 mCWT = 1943;
 mDBG = 2060;
 mENU = 3062;
 mESR = 3189;
 mETH = 3205;
 mFBT = (Ord('F') - Ord('A')) * 26 * 26 + (Ord('B') - Ord('A')) * 26 + (Ord('T') - Ord('A'));
 mFPL = 3781;
 mIAD = 5411;
 mICR = 5477;
 mIDN = 5499;
 mLBT = 7481;
 mLDN = (Ord('L') - Ord('A')) * 26 * 26 + (Ord('D') - Ord('A')) * 26 + (Ord('N') - Ord('A'));
 mLIC = 7646;
 mLIV = (Ord('L') - Ord('A')) * 26 * 26 + (Ord('I') - Ord('A')) * 26 + (Ord('V') - Ord('A'));
 mLIW = (Ord('L') - Ord('A')) * 26 * 26 + (Ord('I') - Ord('A')) * 26 + (Ord('W') - Ord('A'));
 mLWN = (Ord('L') - Ord('A')) * 26 * 26 + (Ord('W') - Ord('A')) * 26 + (Ord('N') - Ord('A'));
 mMBS = (Ord('M') - Ord('A')) * 26 * 26 + (Ord('B') - Ord('A')) * 26 + (Ord('S') - Ord('A'));
 mMTD = 8609;
 mMSV = 8601;
 mPFT = 10289;
 mPRS = (Ord('P') - Ord('A')) * 26 * 26 + (Ord('R') - Ord('A')) * 26 + (Ord('S') - Ord('A'));
 mPRT = 10601;
 //mPRX = 10605;
 mPST = 10627;
 mRBT = 11537;
 mQAF = (Ord('Q') - Ord('A')) * 26 * 26 + (Ord('A') - Ord('A')) * 26 + (Ord('F') - Ord('A'));
 mRES = 11614;
 mSTP = 12677;
 mTAR = 12861;
 mTAS = 12862;
 mTAV = 12865;
 mTDD = 12925;
 mTST = 13331;
 mUNT = (Ord('U') - Ord('A')) * 26 * 26 + (Ord('N') - Ord('A')) * 26 + (Ord('T') - Ord('A'));
 mVAL = 14207;
 mWFP = (((( (ord('W')-ord('A'))*26)+(ord('F')-ord('A')))*26)+(ord('P')-ord('A')));
 mWMD = 15187;
 mZST = 17387;


procedure Tdata5100.dbase_to_slist(list: Tstringlist);
var
  temp, i : integer;
  bdx : TBDXPort;

  procedure dbase_serial_to_slist(list : TStringList; const bdx : TBDXPort);
  begin
      list.add(Format('BDX%d,%d,%d,%d,%d,%d,%d,%d,%d',
                      [Integer(bdx),
                       portData14[bdx].use,
                       portData14[bdx].baud+1,
                       portData14[bdx].parity,
                       portData14[bdx].databits+7,
                       portData14[bdx].stopbits+1,
                       portData14[bdx].termEn,
                       portData14[bdx].dtrEn,
                       portData14[bdx].rs232En]));
  end;

begin
  list.clear();
  list.add(format('IDN"%s"',[id]));
  list.add(format('ADR%d',[netAddress]));

  if hi_res then temp := 1 else temp := 0;
  list.add(format('IAD1,%d,%d,%d,%d,%d,%d',[fs1,dp_pos,res1+1,temp,addtare,senseLine]));
  list.add(format('IAD2,%d,%d,%d,%d,%d,%d',[fs2,dp_pos,res2+1,temp,addtare,senseLine]));
  list.add(format('CWT%d',[cal_weight]));
  list.add(format('WMD%d,%d',[weighing_mode+1,trade_mode]));
  list.add(format('ENU%d',[units]));
  list.add(format('ICR%d',[filter]));
  list.add(format('ASF%d,%d',[av_setting,jitter]));
  list.add(format('LBT0,%d',[z_but]));
  list.add(format('LBT1,%d',[t_but]));
  list.add(format('LBT2,%d',[g_but]));
  list.add(format('LBT3,%d',[f_but[1]]));
  list.add(format('LBT4,%d',[f_but[2]]));
  list.add(format('LBT5,%d',[f_but[3]]));
  list.add(format('MTD%d',[motion]));
  list.add(format('PFT"%s"',[pr_fmt]));
  list.add(format('AFT"%s"',[au_fmt]));

  for i := 0 to NUM_HEADER_LINES-1 do begin
    list.add(format('PST%d,"%s"',[i+1,prhdr[i]]));
//    ShowMEssage(format('PST%d,"%s"',[i+1,prhdr[i]]));
  end;
  list.add(format('PRS,%d,%d,%d,%d,%d,%d,%d,%d,%d', [pr_mode+1, auto_type+1, tic_col, tic_row, auto_src+1, print_interlock, auto_start_char, auto_end_char1, auto_end_char2]));
  list.add(format('AFT"%s"',[au_fmt]));

  list.add(format('ZST%d,%d,%d,%d',[zstartup,ztrac,zrange+1,zband]));

  for i := 1 to NUM_FRONT_FUNC do
  begin
    list.add(Format('FBT%d,%d', [i, front_func[i]]));
  end;

  for i := 1 to NUM_REMOTE_INPUTS do
  begin
    list.add(format('RBT%d,%d', [i, rem[i]]));
  end;

  list.add(format('BUZ%d',[ord(buzEn)]));

  list.add(format('ETH%d,"%s","%s","%s","%s","%s","%s","%s","%s","%s"',[ord(ethDHCPEn),ethAddr,ethMask,ethGateway,ethDNS1,ethDNS2,ethMac,ethHostname,ethSearchDomain,ethDNS3]));

  for bdx := Low(TBDXPort) to High(TBDXPort) do
  begin
    i := Integer(bdx);
    case bdx of
      BDXUnused1, BDXUnused2, BDXUnused3, BDXUnused4:
        ;
      BDXTcpNet:
        list.add(format('BDX%d,,,,,,,,,%d,%d', [i, portData14[bdx].tcpPort, portData14[bdx].tcpTimeout]));
      BDXTcpAuto:
        list.add(format('BDX%d,%d,,,,,,,,%d,%d', [i, portData14[bdx].use, portData14[bdx].tcpPort, portData14[bdx].tcpTimeout]));
      BDXUdpI1, BDXUdpI2:
        list.add(format('BDX%d,%d,,,,,,,,%d', [i, portData14[bdx].use, portData14[bdx].tcpPort]));
      BDXUdpO1, BDXUdpO2:
        list.add(Format('BDX%d,%d,,,,,,,,%d,,"%s"', [i, portData14[bdx].use, portData14[bdx].tcpPort, portData14[bdx].udpDestIP]));
      BDXSerial1, BDXSerial2:
        begin
          if indicator_model_subtype_has_db9_serial(self.model_subtype) then
            dbase_serial_to_slist(list, bdx);
        end;
    else
      dbase_serial_to_slist(list, bdx);
    end;
  end;

  for i := 0 to NUM_SETPOINTS-1 do begin
    list.add(format('LIV%d,%d,%d,,%d,%d,%d,%d,%d,%d,%d,%d',
      [i+1,setp_data[i].setp_type,
           setp_data[i].setp_src+1,
           setp_data[i].setp_target,
           setp_data[i].setp_inflight,
           setp_data[i].setp_hysteresis,
           setp_data[i].setp_logic+1,
           setp_data[i].setp_lock,
           setp_data[i].setp_alarm,
           setp_data[i].setp_timing,
           setp_data[i].setp_reset]));
  end;

  list.add(format('LIW%d,%d,%d,%d,%d,%d',
                  [setp_opt_jog_on,
                   setp_opt_jog_off,
                   setp_opt_jogs_in_set,
                   setp_opt_max_jog_sets,
                   setp_opt_feeder_type,
                   setp_opt_delay_check]));

  list.add(format('MBS%d,%d',[mbEndian, mbPort]));

  list.Add(format('AOC%d,%d,%d,%d,%d,%d,%d',[anlType, anlSource, anlZero,
  anlSpan, anlFrc, anlAbs, anlClip]));

  list.Add(format('CLK%d,%d,%d,%d,%d,%d',[clkHour, clkMinute, clkSecond, clkDay, clkMonth, clkYear]));
  list.Add(format('CFM%d,%d',[time_format, date_format]));

  list.Add(format('QAF%d,%d,%d,%d',[qafEnable, qafDay, qafMonth, qafYear]));

  list.Add(format('BRT%d',[brightness]));

  list.Add(format('UNT%d,%d,%d,%d', [secondary_units, tertiary_units, conversion_factor, show_lboz]));
end;


procedure Tdata5100.recipe_to_slist(all_recipes: boolean; list: Tstringlist);
begin
end;

procedure Tdata5100.get_settings(all_settings: boolean; cmd_list, reply_list: Tstringlist);
var
  i: integer;
  bdx : TBDXPort;
begin
  cmd_list.clear();
  reply_list.clear();

  cmd_list.add('IDN?'); reply_list.add('IDN');
  cmd_list.add('ADR?'); reply_list.add('ADR');
  cmd_list.add('AOC?'); reply_list.add('AOC');
  cmd_list.add('IAD?1'); reply_list.add('IAD');
  cmd_list.add('IAD?2'); reply_list.add('IAD');
  cmd_list.add('WMD?'); reply_list.add('WMD');
  cmd_list.add('ENU?'); reply_list.add('ENU');
  cmd_list.add('ICR?'); reply_list.add('ICR');
  cmd_list.add('LBT?0'); reply_list.add('LBT0,');
  cmd_list.add('LBT?1'); reply_list.add('LBT1,');
  cmd_list.add('LBT?2'); reply_list.add('LBT2,');
  cmd_list.add('LBT?3'); reply_list.add('LBT3,');
  cmd_list.add('LBT?4'); reply_list.add('LBT4,');
  cmd_list.add('LBT?5'); reply_list.add('LBT5,');
  cmd_list.add('MBS?'); reply_list.add('MBS');
  cmd_list.add('MTD?'); reply_list.add('MTD');
  cmd_list.add('ZST?'); reply_list.add('ZST');
  cmd_list.add('CWT?'); reply_list.add('CWT');
  cmd_list.add('LDN?'); reply_list.add('LDN');
  cmd_list.add('LWN?'); reply_list.add('LWN');
  cmd_list.add('WFP?'); reply_list.add('WFP');
  cmd_list.add('CLK?'); reply_list.add('CLK');
  cmd_list.add('CFM?'); reply_list.add('CFM');
  cmd_list.add('QAF?'); reply_list.add('QAF');
  cmd_list.add('BRT?'); reply_list.add('BRT');
  cmd_list.add('UNT?'); reply_list.add('UNT');
  if all_settings then begin
    cmd_list.add('ASF?'); reply_list.add('ASF');
    cmd_list.add('ETH?'); reply_list.add('ETH');
    cmd_list.add('BUZ?'); reply_list.add('BUZ');

    for bdx := Low(TBDXPort) to High(TBDXPort) do
    begin
      case bdx of
        BDXUnused1, BDXUnused2, BDXUnused3, BDXUnused4:
          ;
        BDXSerial1, BDXSerial2:
          begin
            if indicator_model_subtype_has_db9_serial(self.model_subtype) then
            begin
              i := Integer(bdx);
              cmd_list.add(Format('BDX?%d', [i])); reply_list.add('BDX');
            end;
          end;
      else
        i := Integer(bdx);
        cmd_list.add(Format('BDX?%d', [i])); reply_list.add('BDX');
      end;
    end;
    cmd_list.add('PFT?'); reply_list.add('PFT');
    for i := 1 to NUM_HEADER_LINES do begin
      cmd_list.add('PST?' + inttostr(i)); reply_list.add('PST' + inttostr(i) + ',');
    end;
    cmd_list.add('PRS?'); reply_list.add('PRS');
    cmd_list.add('AFT?'); reply_list.add('AFT');

    for i := 1 to NUM_FRONT_FUNC do
    begin
      cmd_list.add(Format('FBT?%d', [i])); reply_list.add(Format('FBT%d,', [i]));
    end;

    for i := 1 to NUM_REMOTE_INPUTS do
    begin
      cmd_list.add(format('RBT?%d', [i])); reply_list.add(format('RBT%d,', [i]));
    end;

    for i := 1 to NUM_LIN_PTS do begin
      cmd_list.add(format('LIC?%d',[i])); reply_list.add(format('LIC%d,',[i]));
    end;

    for i:= 0 to NUM_SETPOINTS-1 do begin
      cmd_list.add(format('LIV?%d',[i+1])); reply_list.add('LIV');
    end;

    cmd_list.add('LIW?'); reply_list.add('LIW');
  end;
end;

procedure Tdata5100.get_recipe(all_recipes: boolean; cmd_list, reply_list: Tstringlist);
begin
end;

procedure Tdata5100.slist_to_dbase(list: Tstringlist);
var
  i: integer;
  parsed_data: Tcommsobj;
  s: string;
  cmd: string;

function cmd_tag(): integer;
var
  tmp, i: integer;
begin
  tmp := 0;
  for i:= 1 to 3 do begin
    tmp := tmp * 26;
    tmp := tmp + ord(cmd[i]) - ord('A');
  end;
  result := tmp;
end;

procedure convert_data();
var
   temp, r,t,dp: integer;
   bdx : TBDXPort;
begin
 with parsed_data do
  case cmd_tag() of
    mLDN: begin
            getIntParam(0, directZero, temp, 32000, -32000);
          end;
    mLWN: begin
            getIntParam(0, directSpan, temp, 32000, -32000);
          end;
    mIDN: begin
            getStrParam(0, id, 10, 0);
            getStrParam(1, serial, 10, 1);
            getStrParam(2, version, 6, 1);
            getStrParam(3, model, 20, 1);
            getIntParam(4, model_subtype, temp, High(Integer), 0);
          end;
    mAFT: getStrParam(0,au_fmt,20,0);
    mIAD: if getIntParam(0,temp,dp,2,1) then begin
            if temp = 1 then begin
               getIntParam(1,fs1,dp,999999,0);
               getIntParam(2,dp_pos,dp,5,0);
               if getIntParam(3,res1,dp,7,1) then dec(res1);
               getIntParam(4,temp,dp,1,0); hi_res := (temp = 1);
               getIntParam(5,addTare,dp,999999,0);
               getIntParam(6,senseLine,dp,1,0);
             end else begin
               getIntParam(1,fs2,dp,999999,0);
               if getIntParam(3,res2,dp,7,1) then dec(res2);
             end;
          end;

     mMTD: getIntParam(0,motion,dp,15,0);

     mASF: begin
             getIntParam(0,av_setting,dp,14,0);
             getintParam(1,jitter,dp,2,0);
           end;

     mENU: getIntParam(0,units,dp,6,0);


     mCWT: getIntParam(0,cal_weight,dp,999999,0);

     mICR: getIntParam(0,filter,dp,100,10);

     mADR: getIntParam(0,netAddress,dp,31,0);

     mWFP:
       begin
         getIntParam(0, temp, dp, 9999, 0);
         CurrentViewerPort := temp;
       end;

     mBDX:
       begin
         // get port id
         if getIntParam(0, t, dp, Integer(High(TBDXPort)), Integer(Low(TBDXPort))) then
         begin
           bdx := TBDXPort(t);
           case bdx of
             BDXTcpNet, BDXTcpAuto:
               begin
                 getIntParam(1, portData14[bdx].use, dp, 5, 0);                 // Only for BDXTcpAuto, but won't hurt to set for BDXTcpNet
                 getIntParam(9, portData14[bdx].tcpPort, dp, 65535, 0);
                 getIntParam(10, portData14[bdx].tcpTimeout, dp, 86400, 0);
               end;
             BDXUdpO1, BDXUdpO2:
               begin
                 getIntParam(1, portData14[bdx].use, dp, 5,  0);
                 getIntParam(9, portData14[bdx].tcpPort, dp, 65535, 0);
                 getStrParam(11, portData14[bdx].udpDestIP, 15, 7);
               end;
             BDXUdpI1, BDXUdpI2:
               begin
                 getIntParam(1, portData14[bdx].use, dp, 5,  0);
                 getIntParam(9, portData14[bdx].tcpPort, dp, 65535, 0);
               end;
           else
             getIntParam(1, portData14[bdx].use,      dp, 9,  0);
             getIntParam(2, portData14[bdx].baud,     dp, 10, 1); portData14[bdx].baud := portData14[bdx].baud - 1;
             getIntParam(3, portData14[bdx].parity,   dp, 2,  0);
             getIntParam(4, portData14[bdx].databits, dp, 8,  7); portData14[bdx].databits := portData14[bdx].databits - 7;
             getIntParam(5, portData14[bdx].stopbits, dp, 2,  1); portData14[bdx].stopbits := portData14[bdx].stopbits - 1;
             getIntParam(6, portData14[bdx].termEn,   dp, 1,  0);
             getIntParam(7, portData14[bdx].dtrEn,    dp, 1,  0);
             getIntParam(8, portData14[bdx].rs232En,  dp, 1,  0);
           end;
         end;
       end;

     mBUZ:
       begin
         getIntParam(0,temp,dp,1,0); buzEn := boolean(temp);
       end;

     mETH:
       begin
         getIntParam(0,temp,dp,1,0); ethDHCPEn := boolean(temp);
         getStrParam(1,ethAddr,15,7);
         getStrParam(2,ethMask,15,7);
         getStrParam(3,ethGateway,15,7);
         getStrParam(4,ethDNS1,15,7);
         getStrParam(5,ethDNS2,15,7);
         getStrParam(6,ethMac,23,7);
         getStrParam(7,ethHostname,64,0);
         getStrParam(8,ethSearchDomain,255,0);
         getStrParam(9,ethDNS3,15,7);
       end;

     mWMD:
       begin
         if getIntParam(0,weighing_mode,dp,3,1) then dec(weighing_mode);
         getIntParam(1,trade_mode,dp,2,0);
       end;

     mLBT:
       if (getIntParam(0,temp,dp,5,0)) then begin
         case temp of
           0: getIntParam(1,z_but,dp,2,0);
           1: getIntParam(1,t_but,dp,2,0);
           2: getIntParam(1,g_but,dp,2,0);
           3: getIntParam(1,f_but[1],dp,2,0);
           4: getIntParam(1,f_but[2],dp,2,0);
           5: getIntParam(1,f_but[3],dp,2,0);
         end;
       end;

     mPFT: begin
             getStrParam(0,pr_fmt,100,0);
           end;
     mLIC: if (getIntParam(0, temp, dp, NUM_LIN_PTS, 1)) then
           begin
               dec(temp);
               if getintParam(1, r, dp, 150, -150) then
               begin
                   if getintParam(2, t, dp, 999999, -99999) then
                   begin
                       lin_pc[temp] := r;
                       lin_cor[temp] := t;
                   end;
               end;
           end;

     mPST: begin
             if (getIntParam(0,temp,dp,6,1)) then
               getStrParam(1,prhdr[temp-1],30,0);
           end;

     mPRS: begin
            //getIntParam(0,ser2_mode,dp,3,0);
            if getIntParam(1,pr_mode,dp,8,1) then dec(pr_mode);
            if getIntParam(2,auto_type,dp,6,1) then dec(auto_type);
            getIntParam(3,tic_col,dp,20,0);
            getIntParam(4,tic_row,dp,10,0);
            if getIntParam(5,auto_src,dp,4,1) then dec(auto_src);
            getIntParam(6, print_interlock, dp, 2, 0);
            getIntParam(7, auto_start_char, dp, 255, 0);
            getIntParam(8, auto_end_char1, dp, 255, 0);
            getIntParam(9, auto_end_char2, dp, 255, 0);
           end;

      mZST: begin
              getIntParam(0,zstartup,dp,1,0);
              getIntParam(1,ztrac,dp,15,0);
              if getIntParam(2,zrange,dp,4,1) then dec(zrange);
              getIntParam(3,zband,dp,999999,0);
            end;
      mFBT: if (getIntParam(0, temp, dp, NUM_FRONT_FUNC, 1)) then
            begin
              getIntParam(1, front_func[temp], dp, 14, 0);
            end;
      mRBT: if (getIntParam(0, temp, dp, NUM_REMOTE_INPUTS, 1)) then
            begin
              getIntParam(1, rem[temp], dp, 22, 0);
            end;
      mMBS: begin
             getIntParam(0,mbEndian,dp,1,0);
             getintParam(1,mbPort,dp,65535,502);
           end;
      mAOC: begin
             getIntParam(0,anlType,dp,1,0);
             getIntParam(1,anlSource,dp,3,0);
             getIntParam(2,anlZero,dp,65535,0);
             getIntParam(3,anlSpan,dp,65535,0);
             getIntParam(4,anlFrc,dp,2,0);
             getIntParam(5,anlAbs,dp,1,0);
             getIntParam(6,anlClip,dp,1,0);
           end;
      mCLK: begin
             getIntParam(0,clkHour,dp,23,0);
             getIntParam(1,clkMinute,dp,59,0);
             getIntParam(2,clkSecond,dp,59,0);
             getIntParam(3,clkDay,dp,31,0);
             getIntParam(4,clkMonth,dp,12,1);
             getIntParam(5,clkYear,dp,99,0);
           end;
      mCFM: begin
             getIntParam(0, time_format, dp, 1, 0);
             getIntParam(1, date_format, dp, 5, 0);
            end;
      mQAF: begin
             getIntParam(0,qafEnable,dp,1,0);
             getIntParam(1,qafDay,dp,2097,0);
             getIntParam(2,qafMonth,dp,12,1);
             getIntParam(3,qafYear,dp,31,1);
           end;
      mBRT: begin
             getIntParam(0,brightness,dp,100,0);
           end;

       mLIV:
         if (getIntParam(0,r,dp,NUM_SETPOINTS, 1)) then begin
           r := r - 1;
           getIntParam(1,setp_data[r].setp_type,dp,11,0);
           if getIntParam(2,setp_data[r].setp_src,dp,2,1) then dec(setp_data[r].setp_src);
           // Param 3 (direction) is no longer used
           getIntParam(4, setp_data[r].setp_target, dp, 999999, -99999);
           getIntParam(5, setp_data[r].setp_inflight, dp, 999999, 0);
           getIntParam(6, setp_data[r].setp_hysteresis, dp, 999999, 0);
           if getIntParam(7,setp_data[r].setp_logic,dp,2,1) then dec(setp_data[r].setp_logic);
           getIntParam(8,setp_data[r].setp_lock,dp,1,0);
           getIntParam(9,setp_data[r].setp_alarm,dp,4,0);
           getIntParam(10, setp_data[r].setp_timing, dp, 3, 0);
           getIntParam(11, setp_data[r].setp_reset, dp, 2, 0);
         end;

      mLIW: begin
              getIntParam(0, setp_opt_jog_on, dp, 60000, 100);
              getIntParam(1, setp_opt_jog_off, dp, 60000, 100);
              getIntParam(2, setp_opt_jogs_in_set, dp, 20, 1);
              getIntParam(3, setp_opt_max_jog_sets, dp, 20, 0);
              getIntParam(4, setp_opt_feeder_type, dp, 1, 0);
              getIntParam(5, setp_opt_delay_check, dp, 60000, 0);
            end;

      mUNT: begin
              getIntParam(0, secondary_units, dp, 6, 0);
              getIntParam(1, tertiary_units, dp, 5, 0);
              getIntParam(2, conversion_factor, dp, 999999, 1);
              getIntParam(3, show_lboz, dp, 1, 0);
            end;
    end;
end;


begin
  parsed_data := Tcommsobj.create();
  try
    for i := 0 to list.count-1 do begin
       s := list.strings[i];
       cmd := s;
       setlength(cmd,3);
       parsed_data.ParseStr(s,FALSE,TRUE);
       convert_data();
    end;
  finally
    parsed_data.destroy();
  end;
end;


destructor Tdata5100.destroy;
begin
end;


constructor Tdata5100.create;
var
 i: integer;
 bdx: TBDXPort;
begin
   dp_pos := 0;
   weighing_mode := 0;
   trade_mode := 0;

   dp_pos := 0;
   fs1 := 4000;
   fs2 := 6000;
   cal_weight:= 3000;
   senseLine := 0;
   res1 := 0; res2 := 1;
   hi_res:= false;
   addTare := 0;
   directZero := 0;
   directSpan := 20000;
   units:= 2;
   av_setting := 0;
   jitter := 0;
   filter := 50;
   motion := 1;
   ztrac := 0;
   zrange := 3;
   zband := 0;
   zstartup := 0;
   for i := 0 to NUM_LIN_PTS-1 do
   begin
     lin_pc[i] := 0;
     lin_cor[i] := 0;
   end;

   id := '';
   serial := '';
   version := '';
   model := '';
   model_subtype := IMST_UNKNOWN;
   for i := 0 to NUM_HEADER_LINES-1 do prhdr[i] := '';
   pr_fmt := '';
   au_fmt := '';

   //ser2_mode := 0;
   pr_mode := 0;
   auto_type := 0;
   auto_src := 0;
   print_interlock := 0;
   tic_col := 0;
   tic_row := 0;
   auto_start_char := $02; // STX
   auto_end_char1 := $03;  // ETX
   auto_end_char2 := 0;

   z_but := 1; t_but := 1; g_but := 1;

   for i := 1 to NUM_FRONT_FUNC do
   begin
     f_but[i] := 1;
     front_func[i] := 0;
   end;

   for i := 1 to NUM_REMOTE_INPUTS do
     rem[i] := 0;

   for i := 0 to NUM_SETPOINTS-1 do begin
     setp_data[i].setp_type := 0;
     setp_data[i].setp_src := 0;
     setp_data[i].setp_logic := 0;
     setp_data[i].setp_lock := 0;
     setp_data[i].setp_alarm := 0;
     setp_data[i].setp_timing := 0;
     setp_data[i].setp_reset := 0;

     setp_data[i].setp_inflight := 0;
     setp_data[i].setp_hysteresis := 0;
     setp_data[i].setp_target := 0;
   end;
   setp_opt_jog_on := 0;
   setp_opt_jog_off := 0;
   setp_opt_jogs_in_set := 0;
   setp_opt_max_jog_sets := 0;
   setp_opt_feeder_type := 0;
   setp_opt_delay_check := 0;

   ethDHCPEn := TRUE;
   ethAddr := '192.168.1.254';
   ethMask := '255.255.255.255';
   ethGateway := '192.168.1.1';
   ethDNS1 := '192.168.1.1';
   ethDNS2 := '0.0.0.0';
   ethDNS2 := '0.0.0.0';
   ethMac := '0.0.0.0.0.0';
   ethHostname := '';
   ethSearchDomain := '';
   mbPort := 0;
   mbEndian := 0;
   anlType := 0;
   anlSource := 0;
   anlFrc := 0;
   anlAbs := 0;
   anlClip := 0;
   anlZero := 0;
   anlSpan := 0;
   clkHour := 0;
   clkMinute := 0;
   clkSecond := 0;
   clkDay := 0;
   clkMonth := 0;
   clkYear := 0;
   time_format := 0;
   date_format := 0;
   qafEnable := 0;
   qafYear := 0;
   qafMonth := 0;
   qafDay := 0;
   brightness := 0;
   secondary_units := 0;
   tertiary_units := 0;
   conversion_factor := 0;
   show_lboz := 0;

   for bdx := Low(TBDXPort) to High(TBDXPort) do
   begin
     portData14[bdx].use := 0;
     portData14[bdx].baud := 5;         // 9600
     portData14[bdx].parity := 0;       // n
     portData14[bdx].databits := 1;     // 8
     portData14[bdx].stopbits := 0;     // 1
     portData14[bdx].termEn := 0;
     portData14[bdx].dtrEn := 0;
     portData14[bdx].rs232En := 1;
     portData14[bdx].tcpPort := 0;
     portData14[bdx].tcpTimeout := 0;

   end;
   CurrentViewerPort := 0;
end;


procedure TData5100.clear_setp(setp: integer; all: boolean);
var
   first, last, i: integer;
begin
  if all then begin
    first := 0;
    last := NUM_SETPOINTS-1;
  end else begin
     first := setp;
     last := setp;
  end;

  for i := first to last do begin
     setp_data[i].setp_type := 0;
     setp_data[i].setp_src := 0;
     setp_data[i].setp_logic := 0;
     setp_data[i].setp_lock := 0;

     setp_data[i].setp_inflight := 0;
     setp_data[i].setp_hysteresis := 0;
     setp_data[i].setp_target := 0;
  end;
end;


function Tdata5100.cal_change(list: Tstringlist): boolean;
var
  cmd: string;
  i: integer;

function cmd_tag(): integer;
var
  tmp, i: integer;
begin
  tmp := 0;
  for i:= 1 to 3 do begin
    tmp := tmp * 26;
    tmp := tmp + ord(cmd[i]) - ord('A');
  end;
  result := tmp;
end;

begin
  result := false;
  for i := 0 to list.count-1 do begin
     cmd := list.strings[i];
     case cmd_tag() of
        mIAD,mWMD,mENU,mICR,mLBT,mMTD,mZST,mCWT: begin
             result := true;
             exit;
             end;
      end;
   end;
end;

function Tdata5100.Ethernet_GetFieldFromStr(s : string; id : integer; sep: char) : string;
var
  y, i : integer;
begin
  if id < 0 then id := 0;
  result := '';

  i := 0;
  while length(s) > 0 do begin
    // find start and end of the next section
    y := Pos(sep,s);
    if y = 0 then y := length(s)+1;

    if i = id then begin
      // if we need this string
      Delete(s,y,100);
      result := s;
      exit;
    end else begin
      // if we need a future string
      Delete(s,1,y);
    end;
    i := i + 1;
  end;
end;

procedure Tdata5100.Ethernet_GetIpOctetsFromStr(s : String; sep: Char; var oct0, oct1, oct2, oct3 : Integer);
begin
  oct0 := StrToInt(self.Ethernet_GetFieldFromStr(s, 0, sep));
  oct1 := StrToInt(self.Ethernet_GetFieldFromStr(s, 1, sep));
  oct2 := StrToInt(self.Ethernet_GetFieldFromStr(s, 2, sep));
  oct3 := StrToInt(self.Ethernet_GetFieldFromStr(s, 3, sep));
end;

procedure RemoveInvalidChars(var s:string; valid: TCharSet);
var
  i : integer;
begin
  i := 1;
  while i <= length(s) do begin
    if not (s[i] in valid) then Delete(s,i,1);
    i := i + 1;
  end;
end;

function Tdata5100.Ethernet_GetStrFromFields(sa : array of string):string;
var
  i : integer;
begin
  // get rid of invalid chars and make sure that
  for i := 0 to length(sa)-1 do begin
    RemoveInvalidChars(sa[i],['0'..'9']);
    if sa[i] = '' then begin
      result := '';
      exit;
    end;
  end;

  result := sa[0];
  for i := 1 to length(sa)-1 do begin
    result := result + '.' +  sa[i];
  end;
end;

function Tdata5100.GetCurrentViewerPort: Integer;
begin
  Result := FCurrentViewerPort;
end;

procedure Tdata5100.SetCurrentViewerPort(port : Integer);
begin
  FCurrentViewerPort := port;
end;

end.
