unit indicator_model;

interface

const

   // Indicator model subtypes
   IMST_UNKNOWN = 0;
   IMST_UNSET = 1;
   IMST_WE2111_ORIGINAL = 2111 shl 16;
   IMST_C510_ORIGINAL = 510 shl 16;
   IMST_C520_RINLINK = (520 shl 16) or 0;
   IMST_C520_DB9SERIAL = (520 shl 16) or 1;
   IMST_C530_ORIGINAL = 530 shl 16;

function indicator_model_subtype_has_db9_serial(imst : Integer) : boolean;


implementation

{
/** Does the indicator have a status LED */
#define indicator_model_subtype_has_status_led(imst) (imst_c520_rinlink == imst || imst_c530_original == imst)


/** Does the indicator have a front rinlink */
#define indicator_model_subtype_has_front_rinlink(imst) (imst_c520_rinlink == imst || imst_c520_db9serial == imst)


/** Does the indicator have a rear rinlink */
#define indicator_model_subtype_has_rear_rinlink(imst) (imst_c520_rinlink == imst || imst_c530_original == imst)
}


// Does the indicator have DB9 serial ports
function indicator_model_subtype_has_db9_serial(imst : Integer) : boolean;
begin
  Result := (IMST_WE2111_ORIGINAL = imst) or (IMST_C510_ORIGINAL = imst) or (IMST_C520_DB9SERIAL = imst);
end;


{
/** Does the indicator have a display */
#define indicator_model_subtype_has_display(imst) (imst_c530_original != imst)


/** Does the indicator have a keyboard */
#define indicator_model_subtype_has_keyboard(imst) (imst_c530_original != imst)


/** Does the indicator have key LEDs */
#define indicator_model_subtype_has_key_leds(imst) (imst_we2111_original == imst || imst_c510_original == imst)


/** Does the indicator have an F3 key (C520, C530 does, C510 & WE2111 doesn't) */
#define indicator_model_subtype_has_f3_key(imst) \
  (imst_c520_rinlink == imst || imst_c520_db9serial == imst || imst_c530_original == imst)


/** How many front keys does the the indicator model */
#define indicator_model_subtype_num_front_keys(imst) \
  ((imst_we2111_original == imst || imst_c510_original == imst) ? 5 : \
   (imst_c520_rinlink == imst || imst_c520_db9serial == imst || imst_c530_original == imst) ? 6 : 0)
}


end.
