unit kenbitmap2;
// This module was producted because of differences in the way the WinME
//  and W2K handle bitmaps.  TKenBitmap is used instead of normal bitmaps.
// 7/7/2003
// Ken Bray
//  Ranger Instruments

interface

uses graphics, sysutils, classes;

type
  TKenBitmap = class (TObject)
  private           
  public
    data : array of array of word;
    filename : string;

    width:integer;
    height:integer;
    numColours:integer;

    constructor create;
    procedure FillWithValue(value:integer; aTop,aLeft,aWidth,aHeight:integer);
    procedure SaveToFile(afilename:string);
    procedure LoadFromFile(afilename:string);

    procedure SetWidthHeightNumColours(w,h,n:integer);

    procedure CopyData(k:TKenBitmap;x,y:integer);
    procedure CopyBitmap(b: TBitmap;x,y:integer);
    procedure CopyStretch(k:TKenBitmap; targetW, targetH: integer);
    procedure SaveAsBitmap(filename:string);

    function GetData(x,y : integer) : integer;

    property backgroundColour:integer read numColours;

  end;

implementation

uses math;

constructor TKenBitmap.create;
begin
  inherited create;

  Width := 0;
  height := 0;
  numColours := 500;
  filename := '';
end;

procedure TKenBitmap.FillWithValue(value:integer; aTop,aLeft,aWidth,aHeight:integer);
var x,y : integer;
var bottom,right : integer;
begin
  bottom := Min(Height-1,aTop+aHeight-1);
  right := Min (Width-1,aLeft+aWidth-1);

  for y := aTop to bottom do begin
    for x := aLeft to right do begin
      data[x,y] := value;
    end;
  end;
end;

procedure TKenBitmap.SaveToFile(afilename:string);
var
  i:integer;
  str : TFileStream;
begin
  filename := afilename;
  str := TFileStream.create(filename,fmCreate);
  try
    //write dimensions
    str.Write(width,4);
    str.Write(height,4);
    str.Write(numColours,4);

    //write data
    for i:= 0 to Width-1 do begin
      str.write(data[i,0],height*2);
    end;
  finally
    str.Free;
  end;
end;

procedure TKenBitmap.LoadFromFile(afilename:string);
var
  i:integer;
  str : TFileStream;
begin
  filename := afilename;
  str :=  TFilestream.create(filename, fmOpenRead);
  try
    // read dimensions
    str.Read(width,4);
    str.Read(height,4);
    str.Read(numColours,4);

    // set height
    SetWidthHeightNumColours(Width,Height,NumColours);

    for i:= 0 to Width-1 do begin
      str.read(data[i,0],height*2);
    end;
  finally
    str.Free;
  end;
end;

procedure TKenBitmap.SetWidthHeightNumColours(w,h,n: integer);
var i: integer;
begin
  Width := w;
  height := h;
  NumColours := n;
  SetLength(data,w);
  for i:= Low(data) to High(data) do begin
    SetLength(data[i],h*2);
  end;
end;


procedure TKenBitmap.CopyStretch(k:TKenBitmap; targetW, targetH: integer);
var
  x,y : integer;
  xk : integer;
  ws,hs,wd,hd : integer;
  ba : array [0..3000] of integer;
begin
  SetWidthHeightNumColours(targetW,targetH,k.numColours);
  filename := k.filename;

  ws := k.width-1;
  hs := k.height-1;
  wd := Width-1;
  hd := height-1;
  for y := 0 to hd do begin
    ba[y] := y * hs div hd;
  end;
  for x := 0 to wd do begin
    xk := x * ws div wd;
    for y := 0 to hd do begin
      data[x,y] := k.data[xk,ba[y]];
    end;
  end;
end;

function TKenBitmap.GetData(x,y : integer) : integer;
begin
  result := data[x,y];
end;



//////////////////////////////////////////////////////////////////////
// Are these functions below used?
//////////////////////////////////////////////////////////////////////



procedure TKenBitmap.CopyData(k:TKenBitmap;x,y:integer);
var
  xd,yd,xs,ys : integer;
begin
  xs := 0;
  for xd := x to Min(x+k.width-1,width-1) do begin
    ys := 0;
    for yd := y to Min(y+k.height-1,height-1) do begin
      data[xd,yd] := k.data[xs,ys];
      ys := ys+1;
    end;
    xs := xs+1;
  end;
end;

procedure TKenBitmap.CopyBitmap(b: TBitmap;x,y:integer);
var
  xd,yd,xs,ys : integer;
  lin : PByteArray;
begin
  ys := 0;
  for yd := y to Min(y+b.height-1,height-1) do begin
    xs := 0;
    lin := b.ScanLine[ys];
    for xd := x to Min(x+b.width-1,width-1) do begin
      data[xd,yd] := lin[xs];
      xs := xs+1;
    end;
    ys := ys+1;
  end;
end;


procedure TKenBitmap.SaveAsBitmap(filename:string);
var
  bm : TBitmap;
  i,j : integer;
  lin : PByteArray;
begin
  bm := TBitmap.create;
  try
    bm.PixelFormat := pf16bit;
    bm.Width := width;
    bm.Height := height;
    for i := 0 to Height-1 do begin
      lin := bm.ScanLine[i];
      for j := 0 to Width-1 do begin
        lin[j] := data[j,i];
      end;
    end;
    bm.SaveToFile(filename);
  finally
    bm.Free;
  end;
end;

end.
 