unit kenMimicDisp;

// Title: Ken Mimic
// Author: Ken Bray
// Date: 27 May 2003
// Description:
//   TKenMimicDisp implements TKenMimic.  This includes a background image,
//   LCD/LED segments and button presses
//
// Usage:
//   See WE21-510 or TEST-526.
//
// Other notes:

interface

uses Windows, Messages, SysUtils, Classes, Graphics, Controls, dialogs,
  extctrls, kenbutton2, kenlcd2, kenmimic2, jpeg;

type
  TKenMimicDisp = class (TKenMimic)
  protected
    fLCDLeft, fLCDTop, fLCDWidth, fLCDHeight : integer;
  public
    lcd : TKenLCD;
    btns : TKenLcdButtons;
    backgroundFilename : string;
    lcdFilename:string;
    keyFilename:string;

    constructor create(owner : Tcomponent); override;
    destructor destroy; override;
    procedure CalcLCDDimensions;
    procedure DrawBackground();
    procedure DrawLCD(); override;
    procedure Resize(); override;
    //procedure Paint(); override;
  published
    property LCDLeft : integer read fLCDLeft write flcdleft;
    property LCDTop : integer read fLCDTop write fLCDTop;
    property LCDWidth : integer read fLCDWidth write flcdWidth;
    property LCDHeight : integer read fLCDHeight write flcdHeight;
  end;

implementation

constructor TKenMimicDisp.create(owner : Tcomponent);
begin
  inherited create(owner);

  Width := 100;
  height := 100;

  fLCDWidth := 1000;
  fLCDHeight := 1000;

  lcd := TKenLCD.create;
  lcd.onColor := clBlack;
  btns := TKenLCDButtons.create;
end;

destructor TKenMimicDisp.destroy;
begin
  lcd.Free;
  btns.Free;
  inherited destroy;
end;

(*
procedure TKenMimicDisp.Paint();
begin
    inherited Paint;
end;
*)
procedure TKenMimicDisp.DrawBackground();
var
  pic : TGraphic;
begin
  if (backgroundFilename = '') then exit;

  // paint the background image
  if UpperCase(ExtractFileExt(backgroundFilename)) = '.BMP' then pic := TBitmap.create
  else if UpperCase(ExtractFileExt(backgroundFilename)) = '.JPG' then pic := TJPEGImage.create
  else begin
    MessageDlg('Image format not recognised',mtError,[mbOK],0);
    exit;
  end;

  try
    pic.LoadFromFile(backgroundFilename);
    Canvas.StretchDraw(ClientRect,pic);
    CalcLCDDimensions;
  finally
    pic.free;
  end;
  lcd.SetupLCDBackground(fLCDLeft*Width div 1000, fLCDTop*Height div 1000, fLCDWidth*Width div 1000, fLCDHeight*height div 1000, Canvas);

  // setup buttons
  if keyFilename <> '' then
    btns.Setup(keyFilename,self);

  // draw lcd
  if lcdFilename <> '' then DrawLCD;
end;

procedure TKenMimicDisp.CalcLCDDimensions;
begin
  if lcdfilename <> '' then lcd.SetupLCDSize(fLCDWidth*Width div 1000, fLCDHeight*height div 1000, lcdfilename);
end;

procedure TKenMimicDisp.Resize();
begin
  if (backgroundFilename <> '') then begin
    Picture.Bitmap.Width := parent.Width;
    Picture.Bitmap.height := parent.Width;
    DrawBackground;
    DrawLCD;
  end;
  // not required
end;

procedure TKenMimicDisp.DrawLCD();
begin
  // draw lcd on canvas
  if lcdFilename <> '' then begin
    lcd.DrawWithBackground( fLCDLeft*Width div 1000, fLCDTop*Height div 1000,
                            fLCDWidth*Width div 1000, fLCDHeight*height div 1000,
                            canvas);
  end;
end;

end.
