unit logging;

interface


procedure CommsLoggingInit(const filename : String);
procedure CommsLoggingDestroy;
procedure CommsLoggingLogMessage(const s: string);
procedure CommsLoggingLogComms(const port : String; dirn_in : Boolean; const s: string);


implementation

uses classes, sysutils, dialogs;

var
  MessageLogFileStream : TFileStream;


procedure CommsLoggingInit(const filename : String);
begin
  if (not Assigned(MessageLogFileStream)) then
  begin
    try
      // First try to open an existing log file for append
      MessageLogFileStream := TFileStream.Create(filename, fmOpenReadWrite or fmShareDenyWrite);
      MessageLogFileStream.Seek(0, soFromEnd);
    except
      // Failed, try to create new file
      try
        // First create the file (fmCreate won't play with fmShare*), then re-open it
        MessageLogFileStream := TFileStream.Create(filename, fmCreate);
        MessageLogFileStream.Free();
        MessageLogFileStream := TFileStream.Create(filename, fmOpenReadWrite or fmShareDenyWrite);
        MessageLogFileStream.Seek(0, soFromEnd);
        CommsLoggingLogMessage('CommsLoggingInit: Log file created');
      except
        on E: Exception do
        begin
          ShowMessage(Format('Log file %s cannot be opened (%s)', [filename, E.Message]));
          MessageLogFileStream := nil;
        end;
      end;
    end;
    CommsLoggingLogMessage('CommsLoggingInit: Logging started');
  end;
end;


procedure CommsLoggingDestroy;
begin
  if (Assigned(MessageLogFileStream)) then
  begin
    CommsLoggingLogMessage('CommsLoggingDestroy: Logging stopped');
    MessageLogFileStream.Free();
    MessageLogFileStream := nil;
  end;
end;


procedure CommsLoggingLogMessage(const s: string);
var msg : String;
begin
  if (Assigned(MessageLogFileStream)) then
  begin
    msg := FormatDateTime('yyyy-mm-dd hh:mm:ss.zzz', Now()) + ': ' + s + #13#10;
    MessageLogFileStream.Write(PChar(msg)^, Length(msg));
  end;
end;


function FormatLoggingNonPrintables(const s : String) : String;
var ch : Char;
    i : Integer;
begin
  Result := '';
  for i := 1 to Length(s) do
  begin
    ch := s[i];
    if (Ord(ch) < Ord(' ')) or (Ord(ch) > Ord('~')) then
      Result := Result + Format('\x%.02x', [Ord(ch)])
    else
      Result := Result + ch;
  end;
end;


procedure CommsLoggingLogComms(const port : String; dirn_in : Boolean; const s: string);
var msg : String;
    dirn : String;
begin
  if (Assigned(MessageLogFileStream)) then
  begin
    if (dirn_in) then dirn := 'in' else dirn := 'out';
    msg := Format('%s %s: %s', [port, dirn, FormatLoggingNonPrintables(s)]);
    CommsLoggingLogMessage(msg);
  end;
end;


end.
