unit tools;

interface

uses
  classes;

type
  TCharSet = set of Char;

const
  CharsetNumeric = ['0'..'9'];
  CharsetAlphabetUpper = ['A'..'Z'];
  CharsetAlphabetLower = ['a'..'z'];

  CharsetHostname : TCharset = CharsetAlphabetLower + CharsetNumeric + [ '-' ];
  CharsetDomainname : TCharset = CharsetAlphabetLower + CharsetNumeric + [ '-', '.' ];
  CharsetIndicatorId : TCharset = CharsetAlphabetLower + CharsetNumeric + [ '-', '.' ];


{
 the following routines deal with integers as weight values, so an artificial decimal point
is added at a fixed location, but the underlying weight value is kept as an integer
}

function my_strtoint(s: string): integer;
{ converts s to integer but allows '.' to be present }
function fmt_int(n: Integer; dp: Integer; dpchar : Char): String;
{ converts n to a string which contains '.' with dp places after }
function fmt_num(s: String; dp: Integer; dpchar : Char): String;
{ takes a string potentially with '.' and formats it so that there are dp places after '.' }
procedure remove_duplicates(l1,l2: Tstringlist);
{ remopves all strings from l1 that are also in l2}

function str_to_double(s: string): double;


implementation

uses
  SysUtils;


function my_strtoint(s: string): integer;
var
  i: integer;
  val: integer;
  minus: boolean;
begin
  val := 0;
  minus := false;
  for i := 1 to length(s) do begin
    case s[i] of
      '+',' ', '.', ',': ;
      '-': minus := true;
      else if ((ord(s[i]) >= ord('0')) and (ord(s[i]) <= ord('9'))) then
         val := (val * 10 ) + ord(s[i]) - ord('0');
      end;
  end;
  if minus then val := -val;
  result := val;
end;

function fmt_int(n: Integer; dp: Integer; dpchar : Char): String;
var
  tmp: string;
begin
  tmp := '';
  case dp of
    0: tmp := format('%d',[n]);
    1: tmp := format('%.1f',[(n/10)]);
    2: tmp := format('%.2f',[(n /100)]);
    3: tmp := format('%.3f',[(n /1000)]);
    4: tmp := format('%.4f',[(n /10000)]);
    5: tmp := format('%.5f',[(n /100000)]);
  end;
  Result := StringReplace(tmp, '.', dpchar, [rfReplaceAll]);
end;

function fmt_num(s: String; dp: Integer; dpchar : Char): String;
var
  i, pos : integer;
  tmp: string;
begin
  tmp := '';
  pos := 0;
  for i := 1 to length(s) do
  begin
    if (s[i] = '.') or (s[i] = ',') then
    begin
      pos := i;
      break;
    end
    else
      tmp := tmp + s[i];
  end;

  if (tmp = '') then
    tmp := '0';

  if dp > 0 then
  begin
    tmp := tmp + dpchar;
    if (pos = 0) then
    begin
      pos := length(tmp);
    end;
    pos := pos + 1;
    for i := 1 to dp do
    begin
      if pos > length(s) then
        tmp := tmp + '0'
      else
      begin
        tmp := tmp + s[pos];
        pos := pos + 1;
      end;
    end;
  end;
  result := tmp;
end;

function str_to_double(s: string): double;
var
  n,d: integer;
  i: integer;
  minus: boolean;
begin
  minus := false;
  n := 0; d := 0;
  for i := 1 to length(s) do begin
    if s[i] = '-' then
       minus := TRUE
    else if s[i] = '.' then d := 1
    else begin
      d := d * 10;
      n := (n * 10) + ord(s[i]) - ord('0');
    end;
  end;
  if (minus) then n := -n;
  if d = 0 then d := 1;
  result := n/d;
end;


procedure remove_duplicates(l1,l2: Tstringlist);
var
  pos1: integer;
begin

  if (l1.count > 1) and (l2.count > 0) then begin
    pos1 := 0;
    repeat
//    r := l2.indexof(l1.strings[pos]);
      if l2.indexof(l1.strings[pos1]) <> -1 then begin
         l1.delete(pos1);
         end
      else begin
         pos1 := pos1+1;
       end;
     until (pos1 >= l1.count);
   end;
end;


end.
